# -*- coding: utf-8 -*-
"""jalonexercicewims."""
from zope.interface import implements
from Products.Archetypes import public as atpublic
from Products.ATContentTypes.content.document import ATDocumentBase, ATDocumentSchema
from Products.ATContentTypes.content.base import registerATCT
# from Products.CMFCore.utils import getToolByName

from jalon.content import contentMessageFactory as _
from jalon.content.config import PROJECTNAME
from jalon.content.interfaces import IJalonExerciceWims

from urlparse import urlparse

import jalon_utils
import re

import json
from copy import deepcopy

from logging import getLogger
LOG = getLogger('[JalonExerciceWims]')

JalonExerciceWimsSchema = ATDocumentSchema.copy() + atpublic.Schema((
    atpublic.StringField(
        'modele',
        required=False,
        accessor='getModele',
        searchable=False,
        widget=atpublic.StringWidget(
            label=_(u"Modèle CreateExo de Wims")
        )),
    atpublic.StringField(
        'permalink',
        required=False,
        accessor='getPermalink',
        searchable=False,
        widget=atpublic.StringWidget(
            label=_(u"Permalien d'un exercice wims publié")
        )),
    atpublic.StringField(
        'qnum',
        required=False,
        accessor='getQnum',
        searchable=False,
        widget=atpublic.StringWidget(
            label=_(u"Nombre d'exercices de la sélection aléatoire auxquels l'étudiant doit répondre. (dans le cas d'un groupe d'exercices)")
        )),
    atpublic.LinesField(
        "listeIdsExos",
        required=False,
        accessor="getListeIdsExos",
        searchable=False,
        widget=atpublic.LinesWidget(
            label=_(u"Liste de tous les exercices de la sélection aléatoire (dans le cas d'un groupe d'exercices)."),
            visible={'view': 'visible', 'edit': 'invisible'},
        )),
))


class JalonExerciceWims(ATDocumentBase):
    """Un Exercice Wims de Mon Espace."""

    implements(IJalonExerciceWims)
    meta_type = 'JalonExerciceWims'
    schema = JalonExerciceWimsSchema
    schema['description'].required = False
    schema['description'].mode = "r"
    schema['text'].required = False
    schema['text'].mode = "r"

    def getBreadcrumbs(self, edit_mode=False):
        """Fournit le fil d'ariane de l'exercice courant."""
        # LOG.info("----- getBreadcrumbs -----")
        portal_url = self.portal_url.getPortalObject().absolute_url()
        crumbs_list = [{"title" : _(u"Mes ressources"),
                        "icon"  : "fa fa-folder-open",
                        "link" : "%s/mes_ressources" % portal_url},
                       {"title": _(u"Mes exercices WIMS"),
                        "icon" : "fa fa-random",
                        "link" : "%s/mes_ressources/mes_exercices_wims" % portal_url},
                       {"title": self.Title(),
                        "icon" : "fa fa-random",
                        "link" : "%s/view" % self.absolute_url()}]
        if edit_mode:
            crumbs_list.append({"title": "édition",
                                "icon" : "fa fa-edit",
                                "link" : "%s/edit_wims_exercice_form" % self.absolute_url()})

        return crumbs_list

    def getIconClass(self):
        """Return the Exercice icon CSS class."""
        # LOG.info("----- getIconClass -----")
        return "fa fa-random"

    def getEditWimsExerciceMacroName(self):
        """Renvoie le nom de la macro qui permet d'editer l'exercice, en fonction du modele."""
        # LOG.info("----- getEditWimsExerciceMacroName -----")
        edit_wims_exercice_macro_name = "edit_wims_exercice_createxo"
        if self.getModele() == "externe":
            edit_wims_exercice_macro_name = "edit_wims_exercice_externe"
        return edit_wims_exercice_macro_name

    def getVariablesDefaut(self, modele):
        """Liste les valeurs par defaut a definir en fonction du modele d'exercice."""
        # LOG.info("----- getVariablesDefaut -----")
        variables_defaut = {"exercicelibre":{},
                            "qcmsimple":
                                {"enonce"          : "Cochez la(les) bonne(s) réponse(s).",
                                 "bonnesrep"       : "bon choix n°1\nbon choix n°2",
                                 "mauvaisesrep"    : "mauvais choix n°1\nmauvais choix n°2",
                                 "tot"             : "5",
                                 "givetrue"        : "2",
                                 "minfalse"        : "0",
                                 "feedback_general": "",
                                 "feedback_bon"    : "",
                                 "feedback_mauvais": "",
                                 "options"         : "checkbox split",
                                 "accolade"        : "1",
                                 "credits"         : "",
                                 "hint"            : "",
                                 "help"            : "",
                                 },
                            "equation":
                                {"precision"    : "1000",
                                 "param_a"      : "randint(1..20)",
                                 "param_b"      : "randint(1..5)",
                                 "param_c"      : "0",
                                 "param_d"      : "0",
                                 "param_e"      : "0",
                                 "equation"     : "\\a * \\b",
                                 "enonce"       : "Je vend \\a chaussette(s) au prix de \\b euros chacune. Quel est le montant de ma vente ?",
                                 "texte_reponse": "Ma réponse",
                                 "accolade"     : "2",
                                 },
                            "texteatrous":
                                {"type_rep"     : "atext",
                                 "donnees"      : "Le texte ci-dessous est incomplet. Remplissez les trous.\n\
<br/>\nL'Université de Nice fut officiellement instituée par décret du 23 octobre ??1965??.\n\
Néanmoins, ses racines historiques remontent au XVIIème siècle, avec le fameux ??Collegium jurisconsultorum niciensium, Collegius niciensius??\n\
crée en ??1639?? par les Princes de Savoie ; il comprenait un important corps de jurisconsultes et sa notoriété dura jusqu'au rattachement de Nice à la France, en ??1860,1870,1850,1840??.\n\
Au XVIIème siècle, une École de ??Médecine, Couture, Danse, Chimie?? dispensa des enseignements appréciés dans toute l'Europe.\n",
                                 "feedback_general": "",
                                 "feedback_bon"    : "",
                                 "feedback_mauvais": "",
                                 "credits"         : "",
                                 "accolade"        : "1",
                                 "list_order"      : "1",
                                 },
                            "marqueruntexte":
                                {"minmark"  : "1",
                                 "maxmark"  : "8",
                                 "data"     : "{Jack,Jean,Louis,Michel,Pierre} ??et,est?? forain, il \n\
??{tien,tiens} ,tient?? ??un,une?? baraque de tir ??a,à?? la noix de coco.\n\
??Ont,On?? ??trouvent,trouve?? des ??Baraque,Baraques?? à noix de Coco dans ??tous,toutes?? les foires.\n\
Les ??,gens?? ??arrive,arrivent??, ??donne,donnent?? des ??,sous??\n\
??est,et?? ??envoie,envoient?? des ??,boules?? sur une noix de coco \n\
??{poser,posé} ,posée?? en haut d'une ??,colonne??.\n\
Ceux qui ??fait,font?? ??{dégringolé,dégringolée} ,dégringoler?? une noix de coco \n\
??{peu,peut,peux} ,peuvent?? ??{le,les} ,la?? ??{gardée,gardé} ,garder??.\n;\
\n\
\n??{Quel,Quels,Quelles} ,Quelle?? idée ??est-je,ai-je??\n\
??{d'acheté,d'achetée,d'achetés} ,d'acheter?? ??{cept,cette,ces,ce} ,cet?? oiseau ?\n\
L'oiselier me dit : <blockquote>« ??{S'est,Cet} ,C'est?? un ??{mal,malle} ,mâle??.\n\
??Attender,Attendez?? une ??,semaine?? qu'il \n\
??{s'abitue,s'abituent,s'habituent} ,s'habitue??, ??est,et?? il chantera. »</blockquote>\n\
??Hors,Or??, l'oiseau ??sobstine,s'obstine?? ??a,à?? ??ce,se?? \n\
??tait,taire?? et il ??fais,fait?? ??tous,tout?? de ??{traver,travert} ,travers??.\n;\
\n\
\nLes ??désert,déserts?? de ??sables,sable?? ??occupe,occupent?? de\n\
??large,larges?? parties {de la planète,du monde,de la Terre} .\n\
Il n'y ??{pleu,pleus,pleuvent} ,pleut?? presque ??,pas??.<br/>\n\
Très ??peut,peu?? de plantes et ??,d'animaux?? y ??vit,vivent??.\n\
Les ??,dunes?? ??son,sont?? des collines de ??,sable?? que le vent ??à,a??\n\
??{construit,construits,construite} ,construites??.<br/>\n\
Les ??grains,graines?? de ??{certain,certaine,certains} ,certaines??\n\
plantes ??reste,restent?? sous le ??sole,sol?? du désert pendant des années.\n\
??{Ils,Elle} ,Elles?? ??ce,se?? ??met,mettent?? ??a,à?? ??{poussées,poussée,poussés} ,pousser?? dès qu'il y a ??une,un?? orage.\n;",
                                 "pre"      : "Marquez les fautes d'orthographe dans le texte ci-dessous.",
                                 "post"     : "",
                                 "options"  : "split"
                                 },
                            "marquerparpropriete":
                                {"explain"      : "Parmi les joueurs de football ci-dessous, marquez ceux qui sont dans l'équipe \prop.",
                                 "prop"         : "française, italienne, allemande",
                                 "data"         : "Fabien Barthez , francaise\nLilian Thuram , francaise\nClaude Makélélé , francaise\nZinedine Zidane , francaise\nFranck Ribéry , francaise\nThierry Henry , francaise\nDavid Trézéguet , francaise\n\
Gianluigi Buffon , italienne\nMorgan De Sanctis , italienne\nAngelo Peruzzi , italienne\nChristian Abbiati , italienne\nMarco Amelia , italienne\n\
Jens Lehmann , allemande\nOliver Kahn , allemande\nTimo Hildebrand , allemande\nPhilipp Lahm , allemande\nArne Friedrich , allemande\n",
                                 "tot"          : "12",
                                 "mingood"      : "1",
                                 "minbad"       : "4",
                                 "options"      : "split",
                                 "presentation" : "liste"
                                 },
                            "questiontextuelletolerante":
                                {"len"          : "20",
                                 "data"         : "l'intensité est : ??ampère??;\nla tension est : ??volt??;\nla résistance est : ??ohm??;\nla capacité d'un condensateur est : ??farad??. Son symbole est : ??F??;\nl'inductance d'un solénoïde est : ??henri??;\nla puissance dissipée sur une composante est : ??watt??;\nla fréquence est : ??hertz??. Son symbole est : ??Hz??;",
                                 "atype"        : "atext",
                                 "include_good" : "oui",
                                 "words"        : "électricité intensité courant tension capacité puissance fréquence",
                                 "pre"          : "En électricité, l'unité de base pour mesurer ",
                                 "post"         : "."
                                 },
                            "taperlemotassocie":
                                {"size"     : "20",
                                 "words"    : "bambou,bambous\nbijou,bijoux\ncadeau,cadeaux\ncaillou,cailloux\ncheval,chevaux\nchou,choux\nciel,cieux\nclou,clous",
                                 "type_rep" : "atext",
                                 "explain"  : "Tapez le pluriel du nom \\name :",
                                 },
                            "reordonner":
                                {"tot"      : "6",
                                 "size"     : "80x50",
                                 "data"     : "Mercure, Vénus, Terre, Mars, Jupiter, Saturne, Uranus, Neptune",
                                 "explain"  : "Mettre les planètes suivantes du système solaire dans le bon ordre, avec la plus proche du Soleil en premier."
                                 },
                            "correspondance":
                                {"tot"             : "6",
                                 "sizev"           : "50",
                                 "sizer"           : "250",
                                 "sizel"           : "250",
                                 "feedback_general": "",
                                 "data"            : "L'Allemagne, Berlin\nL'Australie, Canberra\nLe Canada, Ottawa\nLa Chine, Beijing\nL'Espagne, Madrid\nLes Etats-Unis, Washington\nLa France, Paris\nLa Grande Bretagne, Londres\nL'Inde, New Delhi\nL'Indonésie, Jakarta\nL'Italie, Rome\nLe Japon, Tokyo\nLa Russie, Moscou",
                                 "explain"         : "Établissez la correspondance entre les pays et leurs capitales.",
                                 "accolade"        : "1",
                                 },
                            "classerparpropriete":
                                {"tot"      : "8",
                                 "max1"     : "4",
                                 "size1"    : "120x120",
                                 "prop"     : "Oiseau, Mammifère",
                                 "data"     : "L'aigle, Oiseau\nL'albatros, Oiseau\nL'alouette, Oiseau\nLe canard, Oiseau\nLe corbeau, Oiseau\nLe faucon, Oiseau\nLe goéland, Oiseau\n\
\nLe lion, Mammifère\nL'éléphant, Mammifère\nLe chat <img src='http://upload.wikimedia.org/wikipedia/commons/thumb/d/da/Black_hills_cat-tochichi.jpg/120px-Black_hills_cat-tochichi.jpg' alt='photo d'un chat'/>, Mammifère\nLe cheval, Mammifère\nLe chien, Mammifère\nLe cochon, Mammifère\nLa vache, Mammifère",
                                 "shuffle"  : "shuffle",
                                 "explain"  : "Classez les animaux ci-dessous selon leurs catégories.",
                                 "post"     : "",
                                 "estun"    : "est un",
                                 "noclass"  : "n'appartient à aucune catégorie",
                                 "feedback_general": ""
                                 },
                            "vraifauxmultiples":
                                {"tot"      : "4",
                                 "mintrue"  : "1",
                                 "minfalse" : "1",
                                 "datatrue" : "À tension égale, le courant passant par un résistor est inversement proportionnel à sa résistance.\nÀ courant égal, la tension sur un résistor est proportionnelle à sa résistance.\nLe courant passant par un résistor est proportionnel à la tension appliquée.\nLa puissance dissipée par un résistor est proportionnelle au carré de la tension appliquée.\nLa puissance dissipée par un résistor est proportionnelle au carré du courant.",
                                 "datafalse": "À tension alternative égale, le courant passant par un condensateur est inversement proportionnel à la capacité.\n\
À tension alternative égale, le courant passant par un condensateur est indépendant de la capacité.\n\
À tension alternative égale, le courant passant par un solénoïde est proportionnel à l'inductance.\n\
À tension continue égale, le courant passant par un condensateur est proportionnel à la capacité.\n\
À tension égale, le courant passant par un résistor est proportionnel à sa résistance.\n\
À tension égale, le courant passant par un résistor est indépendant de sa résistance.\n\
Le courant passant par une diode est proportionnel à la tension appliquée.\n\
La puissance dissipée par un résistor est proportionnelle à la tension appliquée.\n\
La puissance dissipée par un résistor est proportionnelle au courant.\n\
La puissance dissipée par un condensateur est proportionnelle {à,au carré de} la tension appliquée.\n\
La puissance dissipée par un condensateur est proportionnelle {au,au carré du} courant.\n\
La puissance dissipée par un solénoïde idéal est proportionnelle {à,au carré de} la tension appliquée.\n\
La puissance dissipée par un solénoïde idéal est proportionnelle {au,au carré du} courant.",
                                 "options"         : "split",
                                 "explain"         : "Parmi les affirmations suivantes, cochez les phrases vraies.",
                                 "accolade"        : "1",
                                 "feedback_general": "",
                                 "credits"         : "",
                                 "hint"            : "",
                                 "help"            : "",
                                 },
                            "texteatrousmultiples":
                                {"data"             : "Le début officiel de la Seconde Guerre Mondiale est marqué par\n\
??la déclaration de guerre,l'offensive?? de\n\
??la Grande Bretagne et la France,la France,la Grande Bretagne,l'Allemagne,l'Union Soviétique??\n\
contre ??l'Allemagne,la Pologne,l'Union Soviétique,la France??.\n;\n\
La Seconde Guerre Mondiale s'est déroulée entre ??1939?? et ??1945??.\n\
;\n\
??La Grande Bretagne et la France ont,La France a,La Grande Bretagne a,Les États-Unis ont,L'Union Soviétique a?? \n\
déclaré la guerre contre l'Allemagne en ??1939?? à la suite de l'envahissement de ??la Pologne?? par cette dernière.\n\
;\n\
Pendant la Seconde Guerre Mondiale, l'Espagne sous ??Francisco Franco|[F.|Francisco|] Franco?? est un pays\n\
??neutre,axe,allié,envahi??.\n;\n??Après avoir battu,Avant d'attaquer?? la France,\n\
{l'Allemagne,Hitler,l'Allemagne Nazie} a lancé une attaque surprise contre l'URSS en\n\
??décembre,{novembre,octobre},{septembre,août,juillet},{juin,mai,avril},{mars,février,janvier}??\n\
??1940??, sous le nom {du plan,de l'opération} ??Barbarossa??.",
                                 "pre"              : "Completez le texte suivant :",
                                 "post"             : "",
                                 "feedback_general" : "",
                                 "type_rep"         : "atext",
                                 "credits"          : "",
                                 "hint"             : "",
                                 "help"             : "",
                                 },
                             "qcmsuite":
                                {"list_id_questions": "data1 data2",
                                 "instruction"                : "Instruction globale : Répondez à chaque question d'une première série, validez, puis répondez aux questions de la seconde série.",
                                 "alea"                       : "yes",
                                 "nb_questions_max"           : "10",
                                 "nb_etapes"                  : "1",
                                 "pourcentage_validation"     : "0",
                                 "accolade"                   : "1",
                                 "questions"                  : "*-*text{data1=Énoncé de la Question 1\nFeedback affiché une fois la réponse à la question 1 envoyée\n\
1\nProposition 1\nProposition 2\nProposition 3\nProposition 4}\n\n\
:Question 2\n\n*-*text{data2=En quelle année eut lieu la bataille de Marignan ?\n\
Marignan fut la première victoire du jeune roi François Ier, la première année de son règne.\n\
1\n1515\n1414\n1313\n1616}",
                                 "boucle_data_q"              : "*-*text{data_q=\data1!= ? 1:}\n*-*text{data_q=\data2!= ? wims(append item 2 to \data_q)}",
                                 "boucle_battage"             : "*-*text{QUEST=*-*battage[\i]=1? \data1}\n *-*text{QUEST=*-*battage[\i]=2? \data2}",
                                 "boucle_answer"              : "*-*answer{}{\REP1;\CHOIX[1;]}{type=*-*anstype}{option=\option}\n*-*answer{}{\REP2;\CHOIX[2;]}{type=*-*anstype}{option=\option}",
                                 "boucle_matrix_rep"          : "\matrix{REP = \REP1\n\REP2}",
                                 "boucle_condition_questions" : "\condition{Question 1 : Aucune mauvaise réponse }{\TEST[1;3]=0}\n\
                                                                 \condition{Question 1 : Tout juste }{\TEST[1;1]=0 and \TEST[1;2]=0 and \TEST[1;3]=0}\n\
                                                                 \condition{Question 2 : Aucune mauvaise réponse }{\TEST[2;3]=0 and \step >=\CONDSTEP[2]}\n\
                                                                 \condition{Question 2 : Tout juste }{\TEST[2;1]=0 and \TEST[2;2]=0 and \TEST[2;3]=0 and \step >=\CONDSTEP[2]}",
                                 "credits"                    : "",
                                 "columns"                    : "1",
                                 "anstype"                    : "checkbox",
                                 "answer_given"               : "2",
                                 }
                            }
        if modele in variables_defaut:
            return variables_defaut[modele]
        else:
            return None

    def addExoWims(self, idobj=None, title=None, author_id=None, modele=None, form=None, sandbox=False):
        """Ajoute ou modifie un exercice wims."""
        # LOG.info("----- addExoWims (sandbox=%s)-----" % sandbox)
        title = self.formaterTitreWIMS(title)
        member = self.portal_membership.getMemberById(author_id)
        auth_email = member.getProperty("email")
        if not auth_email:
            auth_email = str(member.getProperty("mail"))
        fullname = member.getProperty("fullname")
        if not fullname:
            fullname = member.getProperty("displayName")

        source = str(getattr(self, "modele_%s.oef" % modele))
        # LOG.info("addExoWims / original source=\n%s" % source)
        # La source de départ contient des caracteres html echapés, qu'il faut rétablir avant de les renvoyer à WIMS.
        source = source.replace("&amp;", "&")
        source = source.replace("&lt;", "<")
        source = source.replace("&gt;", ">")
        source = source.replace("&quot;", '"')

        # on souhaite conserver certains '&quot;'
        source = source.replace("$$quot;", '&quot;')

        variables_base = {"author": fullname, "email": auth_email, "title": title}
        variables_modele = self.getVariablesDefaut(modele)
        # ajoute les variables communes aux variables du modele
        variables_modele.update(variables_base)

        # dico contiendra les parametres a envoyer a WIMS
        dico = {"authMember": author_id, "qexo": idobj}
        dico["qclass"] = "%s_1" % self.aq_parent.getComplement()
        # LOG.info("[addExoWims] qclass=%s" % dico["qclass"])
        # Cas d'une creation
        if not form:
            if modele == "groupe":
                return self.ajouterSerie(author_id)

            param = variables_modele
            if modele == "exercicelibre":
                for key in param.keys():
                    source = source.replace("$$%s$$" % key, param[key])
                    param["exercicelibre"] = source
            if param is None:
                # LOG.error("addExoWims / getVariablesDefaut return None")
                return None

        # Cas d'une modification (ou d'un import)
        else:
            # On copie le formulaire pour éviter de le modifier
            param = deepcopy(form)
            if "option" in param:
                # Dans le cas d'une modification, option devrait toujours contenir "force_rewrite" à priori
                # Si "option" n'est pas défini, c'est qu'on fait un import.
                dico["option"] = param["option"]

            if "options" in param and type(param["options"]) is list:
                param["options"] = " ".join(param["options"])
            if modele == "qcmsuite":
                nb_questions = 1
                for key in param.keys():
                    if key.startswith("enonce"):
                        nb_questions = nb_questions + 1
                list_id_questions = ["data1"]

                questions = [self.genererDataQuestionSuite(param["enonce0"], param["feedback0"], param["reponses0"], 1)]

                boucle_data_q = ["*-*text{data_q=\data1!= ? 1:}"]
                boucle_battage = [" *-*matrix{QUEST=*-*battage[\i]=1? \data1}"]
                boucle_answer = ["*-*answer{}{\REP1;\CHOIX[1;]}{type=*-*anstype}{option=\option}"]
                boucle_matrix_rep = ["\matrix{REP = \REP1"]
                boucle_condition_questions = ["\condition{Question 1 : Aucune mauvaise réponse }{\TEST[1;3]=0}", "\condition{Question 1 : Tout juste }{\TEST[1;3]=0 and (\\anstype issametext radio or (\TEST[1;2]=0 and \TEST[1;1]=0))}"]
                for i in range(2, nb_questions):
                    list_id_questions.append("data%s" % str(i))
                    questions.append(":Question %s" % str(i))

                    question_string = self.genererDataQuestionSuite(param["enonce%s" % str(i - 1)],
                                                                    param["feedback%s" % str(i - 1)],
                                                                    param["reponses%s" % str(i - 1)],
                                                                    i)
                    questions.append(question_string)

                    boucle_data_q.append("*-*text{data_q=\data%s!= ? wims(append item %s to \data_q)}" % (str(i), str(i)))
                    boucle_battage.append(" *-*matrix{QUEST=*-*battage[\i]=%s? \data%s}" % (str(i), str(i)))
                    boucle_answer.append("*-*answer{}{\REP%s;\CHOIX[%s;]}{type=*-*anstype}{option=\option}" % (str(i), str(i)))
                    boucle_matrix_rep.append("\REP%s" % str(i))
                    boucle_condition_questions.append("\condition{Question %s : Aucune mauvaise réponse }{\TEST[%s;3]=0 and \step >=\CONDSTEP[%s]}" % (str(i),  str(i), str(i)))
                    boucle_condition_questions.append("\condition{Question %s : Tout juste }{\TEST[%s;3]=0 and (\\anstype issametext radio or (\TEST[%s;2]=0 and \TEST[%s;1]=0)) and \step >=\CONDSTEP[%s]}" % (str(i), str(i),  str(i), str(i), str(i)))

                boucle_matrix_rep[-1] = "%s}" % boucle_matrix_rep[-1]
                param["list_id_questions"] = " ".join(list_id_questions)
                param["questions"] = "\n".join(questions)
                param["boucle_data_q"] = "\n".join(boucle_data_q)
                param["boucle_battage"] = "\n".join(boucle_battage)
                param["boucle_answer"] = "\n".join(boucle_answer)
                param["boucle_matrix_rep"] = "\n".join(boucle_matrix_rep)
                param["boucle_condition_questions"] = "\n".join(boucle_condition_questions)

        self.listeIdsGroupes = [0]

        # wims = getToolByName(self, "portal_wims")
        if modele == "exercicelibre":
            try:
                dico["source"] = param["exercicelibre"].decode("utf-8").encode("iso-8859-1")
            except:
                dico["source"] = param["exercicelibre"]

            if sandbox:
                dico["sandbox"] = True
            # result = json.loads(self.aq_parent.wims("callJob", dico))
            return self.aq_parent.wims("creerExercice", dico)
        else:
            param.update(variables_base)

            for key in variables_modele:
                if key in param:
                    # Le parametre a été envoyé par le formulaire
                    try:
                        param[key].decode("utf-8")
                    except:
                        LOG.info("[addExoWims] - Chaine non UTF-8 envoyee : %s" % key)
                        LOG.info(param[key])

                    try:
                        source = source.replace("$$%s$$" % key, param[key])
                    except UnicodeDecodeError:
                        # typiquement, dans le cas de l'admin par exemple, son nom est en unicode, pas utf-8
                        source = source.replace("$$%s$$" % key, param[key].encode("utf-8"))
                        """print e
                        LOG.info("[addExoWims] - UnicodeDecodeError")
                        LOG.info("[addExoWims] param[%s] = '%s'" % (key, param[key]))
                        #return None"""
                else:
                    # Si la variable n'est pas fournie par le formulaire, on la place à vide
                    # (cas des checkboxes décochées par exemple)
                    source = source.replace("$$%s$$" % key, "")
            # source = source.replace("--nbsp;", "&nbsp;")
            source = source.replace("*-*", "\\")

            # iso-8859-1 ne permet pas d'encoder certains caracteres speciaux comme œ ou €
            # attention : cela ajoute des ";" qui peuvent nuire à certains modèle d'exos (QCM suite par exemple)
            # pour ces derniers, on pourrait convertir chaque champ compatible ?
            if modele not in ["qcmsuite"]:
                source = jalon_utils.convertUTF8ToHTMLEntities(source)

            # try:
            # on ne peux pas empecher la creation d'un exercice pour une erreur d'encodage. on ignore donc les caracteres non reconnus.
            source = source.decode("utf-8").encode("iso-8859-1", "ignore")
            # except:
            #   print "Caractere non reconnu dans creerExercice WIMS (utility.py)"
            #  pass
            # print "utility/creerExercice qexo: %s" % param["qexo"]
            dico["source"] = source

            if sandbox:
                dico["sandbox"] = True
            retour = self.aq_parent.wims("creerExercice", dico)
            # Cas ou on modifie un modele qcm suite
            if form and modele == "qcmsuite":
                retour["list_id_questions"] = list_id_questions
            return retour

    def genererDataQuestionSuite(self, enonce, feedback, reponses, id_question):
        u"""Genere une chaine au format spécifique des questions du modele "QCM Suite"."""
        # LOG.info("----- genererDataQuestionSuite -----")
        enonce = enonce.replace("\n", "<br/>")
        # Si la premiere ligne contient "Qtitle", alors on ne remplace pas le premier saut de ligne
        if enonce.startswith("Qtitle"):
            enonce = enonce.replace("<br/>", "\n", 1)

        feedback = feedback.replace("\n", "<br/>")

        return "*-*text{data%s=asis(%s\n%s\n%s)}" % (str(id_question), enonce, feedback , reponses)

    def ajouterSerie(self, author):
        """Ajoute un groupe d'exercices wims."""
        # LOG.info("----- ajouterSerie -----")
        liste_exos = self.getListeIdsExos()
        if len(liste_exos) > 0:
            # qclass = "%s_1" % self.aq_parent.getComplement()
            # rep = json.loads(self.aq_parent.wims("callJob", {"job": "checksheet", "code": author, "qclass": qclass, "qsheet": "1"}))
            # if rep["status"] == "ERROR":
            #    self.aq_parent.wims("creerFeuille", {"authMember": author, "qclass": qclass, "title": "Feuille de tests des sélections aléatoires", "description": ""})
            for exo_id in liste_exos:
                exo = getattr(self.aq_parent, exo_id)
                if exo.getModele() != "groupe":
                    exo.addRelatedItem(self)

            return {"status": "OK", "code": "GROUP_ADDED"}

            # return self.aq_parent.wims("lierExoFeuille", {"listeExos": self.getListeIdsExos(), "qnum": self.getQnum(), "title": self.Title(), "authMember": author, "qclass": qclass, "qsheet": "1"})
        else:
            return {"status": "ERROR", "err_code": "NO_EXERCICE", "message": _(u"Aucun exercice selectionné")}

    def supprimerSerie(self):
        """Supprime un groupe d'exercices wims."""
        # LOG.info("----- supprimerSerie -----")
        liste_exos = self.getListeIdsExos()
        if len(liste_exos) > 0:
            for exo_id in liste_exos:
                exo = getattr(self.aq_parent, exo_id, None)
                if exo:
                    exo.removeRelatedItem(self)

            return {"status": "OK", "code": "GROUP_ADDED"}
        else:
            return {"status": "ERROR", "code": "NO_EXERCICE"}

    def ajouterTag(self, tag):
        """ajoute Tag."""
        # LOG.info("----- ajouterTag -----")
        return jalon_utils.setTag(self, tag)

    def authUser(self, quser=None, qclass=None, request=None):
        """AuthUser WIMS : permet d'authentifier "quser" dans une classe wims "qclass"."""
        # LOG.info("----- authUser -----")
        return jalon_utils.authUser(self.aq_parent, quser, qclass, request)

    def delExoWims(self):
        u"""Supprime l'exercice wims courant sur le serveur WIMS.

        La suppression coté jalon se fait ensuite dans folder_delete.cpy

        """
        # LOG.info("----- delExoWims -----")
        qclass = "%s_1" % self.aq_parent.getComplement()
        author = self.portal_membership.getAuthenticatedMember().getId()
        qexo = self.getId()
        modele = self.getModele()

        if modele == "groupe":
            retour = self.supprimerSerie()
        elif modele == "externe":
            retour = "TODO"
        else:
            dico = {"job": "delexo", "code": author, "qclass": qclass, "qexo": qexo}
            rep_wims = self.aq_parent.wims("callJob", dico)
            retour = self.aq_parent.wims("verifierRetourWims", {"rep": rep_wims, "fonction": "jalonexercicewims.py/delExoWims", "message": "suppression d'un exo de Mes ressources", "requete": dico})
        return retour

    def lister_modules_wims(self, authMember, module_path="/"):
        u"""Liste tous les modules wims publiés sous le niveau "module_path"."""
        # LOG.info("----- lister_modules_wims -----")
        dico = {"job": "listmodules", "option": module_path, "code": authMember}
        rep_wims = self.aq_parent.wims("callJob", dico)
        return self.aq_parent.wims("verifierRetourWims", {"rep": rep_wims, "fonction": "jalonexercicewims.py/lister_modules_wims", "message": "demande une Liste de modules wims publiés ", "requete": dico})

    def getExoXML(self, formatXML="OLX", version="latest"):
        """Appelle la fonction getExoXML de jalonexportswims."""
        # LOG.info("----- getExoXML -----")
        import jalonexportswims
        return jalonexportswims.getExoXML(context=self, formatXML=formatXML, version=version)

    def getExoZIP(self, filename_path, exo_donnees):
        """Appelle la fonction getExoXML de jalonexportswims."""
        # LOG.info("----- getExoZIP -----")
        import jalonexportswims
        return jalonexportswims.getExoZIP(filename_path=filename_path, exo_donnees=exo_donnees)

    def getListeExports(self):
        u"""Fournit la liste des formats exportables a partir du modèle courant."""
        # LOG.info("----- getListeExports -----")
        liste_formats = {"qcmsimple":                  ["QTI", "OLX", "OEF", "FLL"],
                         "equation":                   ["OEF"],
                         "texteatrous":                ["OEF"],
                         "marqueruntexte":             ["OEF"],
                         "marquerparpropriete":        ["OEF"],
                         "questiontextuelletolerante": ["OEF"],
                         "taperlemotassocie":          ["OEF"],
                         "reordonner":                 ["OEF"],
                         "correspondance":             ["OEF"],
                         "classerparpropriete":        ["OEF"],
                         "vraifauxmultiples":          ["OEF"],
                         "texteatrousmultiples":       ["OEF"],
                         "qcmsuite":                   ["QTI", "OEF"]
                         }
        if self.modele in liste_formats:
            return liste_formats[self.modele]
        else:
            return []

    def getModule(self, authMember, module_path):
        """retourne toutes les infos disponibles pour le module "module_path"."""
        # LOG.info("----- getModule -----")
        dico = {"job": "getmodule", "option": module_path, "code": authMember}
        rep_wims = self.aq_parent.wims("callJob", dico)
        return self.aq_parent.wims("verifierRetourWims", {"rep": rep_wims, "fonction": "jalonexercicewims.py/getModule", "message": "demande les infos d'un module", "requete": dico})

    def getTypeWims(self):
        """Retourne le type d'element (exercice / groupe)."""
        # LOG.info("----- getTypeWims -----")
        if self.modele == "groupe":
            return "Groupe"
        else:
            return "Exercice"

    def cleanData(self, input_data):
        """clean Data."""
        # Il faut verifier quelques points à l'interieur des parametres :
        # 1/ attention aux caracteres speciaux
        # 2/ attention aux }\n qui pourraient etre contenus.
        #   1ere Solution envisagee ==> remplacer "}" par "&#125;" ? ==> non car le code doit etre interprete par wims.
        #   2e solution ==> remplacer "}\n" par  "}\t\n". Les \t ne seront pas affiches en HTML
        # LOG.info("----- cleanData -----")
        input_data = input_data.replace("\r\n", "\n")
        input_data = input_data.replace("\r", "\n")
        input_data = input_data.replace("\n\n", "\n")
        input_data = input_data.replace("  ", " ")
        input_data = input_data.replace("}\n", "}\t\n")
        input_data = input_data.replace("}", "}\t")
        input_data = input_data.replace("\t\t", "\t")

        return input_data

    def getExoOEF(self, modele, authMember, requete={}):
        """permet d'obtenir le code source brut (OEF) d'un exercice WIMS."""
        # LOG.info("----- getExoOEF -----")
        fichier = self.aq_parent.wims("callJob", {"job": "getexofile", "qclass": "%s_1" % self.aq_parent.getComplement(), "qexo": self.getId(), "code": authMember})
        try:
            json.loads(fichier)
            # LOG.error("[getExoOEF] ERREUR WIMS / retour = %s" % retour)
            # Si json arrive a parser la reponse, c'est une erreur. WIMS doit être indisponible (ou mal configuré).
            # autre erreur possible : l'exercice demandé a disparu de WIMS ?

            self.aq_parent.wims("verifierRetourWims", {"rep": fichier,
                                                       "fonction": "jalonexercicewims.py/getExoOEF",
                                                       "message": "Impossible d'obtenir un exo WIMS (demandeur = %s)" % authMember,
                                                       "jalon_request": requete
                                                       })
            message = _(u"Impossible d'obtenir cet exercice. Le serveur WIMS semble temporairement inaccessible. Merci de retenter ultérieurement svp.")
            self.plone_utils.addPortalMessage(message, type='error')
            # [TODO] : ici on pourrait peut-etre renvoyer un dico avec un eventuel code d'erreur pour pouvoir prendr en charge plus de cas.
            return {"request_status": "ERROR", "code_source": None, "error_message": message}
        except:
            # Si "fichier" n'est pas un JSON correct, ce doit bien etre un OEF.
            return {"request_status": "OK", "code_source": fichier}

    def getExoWims(self, modele, authMember, requete={}):
        """Permet de parser le code source d'un exercice WIMS."""
        # LOG.info("[getExoWims] modele = %s" % modele)
        # Il faudra faire un traitement specifique aux exercices externes ici
        if modele == "externe":
            if "permalink" in requete :
                # Cas où on recharge la page de modification (permalien incorrect).
                # LOG.info("[getExoWims] PERMALINK = %s" % requete["permalink"])
                return requete
            else:
                if self.permalink != "":
                    displayed_permalink = "%s?%s&cmd=new" % (self.getUrlServeur(), self.permalink)
                else:
                    displayed_permalink = ""
                return {"status": "OK", "permalink": displayed_permalink}
        # Cas où on recharge la page de modification (un parametre doit etre manquant).
        # on recharge alors simplement le formulaire precedent.
        if "option" in requete and requete["option"] == "force_rewrite":
            requete["options_split"] = 0
            requete["options_eqweight"] = 0
            requete["options_checkbox"] = 0
            if "options" in requete:
                variable = requete["options"]
                if "split" in variable:
                    requete["options_split"] = 1
                if "eqweight" in variable:
                    requete["options_eqweight"] = 1
                if "checkbox" in variable:
                    requete["options_checkbox"] = 1
            return requete

        parsed_exercice = self.getExoOEF(modele, authMember, requete)
        fichier = parsed_exercice["code_source"]

        if fichier is None or modele == "exercicelibre":
            return parsed_exercice
        else:
            variables_parse = {"qcmsimple": {"enonce":           "text{explain",
                                             "bonnesrep":        "matrix{datatrue",
                                             "mauvaisesrep":     "matrix{datafalse",
                                             "tot":              "integer{tot",
                                             "givetrue":         "integer{givetrue",
                                             "minfalse":         "integer{minfalse",
                                             "feedback_general": "text{feedback_general",
                                             "feedback_bon":     "text{feedback_bon",
                                             "feedback_mauvais": "text{feedback_mauvais",
                                             "options":          "text{option",
                                             "accolade":         "text{accolade=item\(",
                                             "credits":          "credits{",
                                             "hint":             "hint{",
                                             "help":             "help{",
                                             },
                               "equation": {"precision":     "precision{",
                                            "equation":      "real{ans",
                                            "enonce":        "text{explain",
                                            "texte_reponse": "answer{",
                                            "accolade":      "text{accolade=item\(",
                                            },
                               "texteatrous": {"type_rep":  "text{type_rep",
                                               "donnees":   "text{data",
                                               "feedback_general": "text{feedback_general",
                                               "feedback_bon"    : "text{feedback_bon",
                                               "feedback_mauvais": "text{feedback_mauvais",
                                               "credits":          "credits{",
                                               "accolade":         "text{accolade=item\(",
                                               "list_order":       "text{list_order=item\(",
                                               },
                               "marqueruntexte": {"minmark": "integer{minmark",
                                                  "maxmark": "integer{maxmark",
                                                  "data":    "text{data",
                                                  "pre":     "text{pre",
                                                  "post":    "text{post",
                                                  "options": "text{option"
                                                  },
                               "marquerparpropriete": {"explain":      "text{explain",
                                                       "prop":         "text{prop",
                                                       "data":         "matrix{data",
                                                       "tot":          "integer{tot",
                                                       "mingood":      "integer{mingood",
                                                       "minbad":       "integer{minbad",
                                                       "options":      "text{option",
                                                       "presentation": "text{presentation"
                                                       },
                               "questiontextuelletolerante": {"len":          "integer{len",
                                                              "data":         "text{data",
                                                              "atype":        "text{atype",
                                                              "include_good": "text{include_good",
                                                              "words":        "text{words",
                                                              "pre":          "text{pre",
                                                              "post":         "text{post"
                                                              },
                               "taperlemotassocie": {"size":     "integer{size",
                                                     "words":    "atrix{data",
                                                     "type_rep": "text{atype",
                                                     "explain":  "text{explain"
                                                     },
                               "reordonner": {"tot":     "integer{tot",
                                              "size":    "text{size",
                                              "data":    "text{data",
                                              "explain": "text{explain"
                                              },
                               "correspondance": {"tot":              "integer{tot",
                                                  "sizev":            "integer{sizev",
                                                  "sizer":            "integer{sizer",
                                                  "sizel":            "integer{sizel",
                                                  "feedback_general": "text{feedback_general",
                                                  "data":             "matrix{data",
                                                  "explain":          "text{explain",
                                                  "accolade":         "text{accolade=item\(",
                                                  },
                               "classerparpropriete": {"tot":              "integer{tot",
                                                       "max1":             "integer{max1",
                                                       "size1":            "text{size1",
                                                       "prop":             "text{prop",
                                                       "data":             "matrix{data",
                                                       "shuffle":          "text{option",
                                                       "explain":          "text{pre",
                                                       "post":             "text{post",
                                                       "estun":            "text{estun",
                                                       "noclass":          "text{noclass",
                                                       "feedback_general": "text{feedback_general"
                                                       },
                               "vraifauxmultiples": {"tot":              "integer{tot",
                                                     "mintrue":          "integer{mintrue",
                                                     "minfalse":         "integer{minfalse",
                                                     "datatrue":         "matrix{datatrue",
                                                     "datafalse":        "matrix{datafalse",
                                                     "options":          "text{option",
                                                     "explain":          "text{explain",
                                                     "accolade":         "text{accolade=item\(",
                                                     "feedback_general": "text{feedback_general",
                                                     "credits":          "credits{",
                                                     "hint":             "hint{",
                                                     "help":             "help{",
                                                     },
                               "texteatrousmultiples": {"data":             "text{data",
                                                        "pre":              "text{pre",
                                                        "post":             "text{post",
                                                        "feedback_general": "text{feedback_general",
                                                        "type_rep":         "text{type_rep",
                                                        "credits":          "credits{",
                                                        "hint":             "hint{",
                                                        "help":             "help{",
                                                        },
                                "qcmsuite": {"instruction"            : "text{instruction",
                                             "alea"                   : "text{alea",
                                             "accolade"               : "text{accolade=item\(",
                                             "nb_questions_max"       : "text{N",
                                             "nb_etapes"              : "text{MAX",
                                             "pourcentage_validation" : "text{percent",
                                             "credits"                : "credits{",
                                             "columns"                : "text{columns",
                                             "anstype"                : "text{anstype",
                                             "answer_given"           : "text{answer_given=item\(",
                                             }
                               }

            if modele == "equation":
                lettres = ["a", "b", "c", "d", "e"]
                for lettre in lettres:
                    variables_parse["equation"]["param_%s" % lettre] = "real{%s" % lettre

            fichier = fichier.replace("\n", "_ENDLINE_")

            for key in variables_parse[modele].keys():
                # Certaines instructions de wims fonctionnent differement (comme "precision")
                if key not in ["precision", "texte_reponse", "accolade", "credits", "hint", "help", "answer_given", "list_order"]:
                    pattern = "%s=(.*?)}_ENDLINE_" % variables_parse[modele][key]
                else:
                    if key in ["precision", "credits", "hint", "help"]:
                        # exemple : \precision{100}
                        pattern = "%s(.*?)}_ENDLINE_" % variables_parse[modele][key]
                    if key == "texte_reponse":
                        # exemple : \answer{Votre reponse est}{\ans}{type=number}
                        pattern = "%s([^\}]*)\}" % variables_parse[modele][key]
                    if key in ["accolade", "answer_given", "list_order"]:
                        # exemple : \text{accolade=item($$variable$$,1 oui,\n2 non)}
                        pattern = "%s([^,]*)" % variables_parse[modele][key]

                m = re.compile(pattern)
                recherche = m.search(fichier)
                # LOG.info("[getExoWims] variable : %s // valeur : %s" % (key, recherche.group(1)))
                if recherche is not None:
                    variable = recherche.group(1)
                    # Au cas ou la chaine "}_ENDLINE_" avait été introduite dans la variable, le nombre d'accolades sera impair.
                    # On continue alors la recherche jusqu'à la prochaine occurence.
                    if key not in ["texte_reponse", "accolade", "answer_given", "list_order"]:
                        variable = self.extendResearch(variable, fichier, recherche.end())

                else:
                    variable = self.getVariablesDefaut(modele)[key]
                if key == "options":
                    if "split" in variable:
                        parsed_exercice["%s_split" % key] = 1
                    else:
                        parsed_exercice["%s_split" % key] = 0
                    if "checkbox" in variable:
                        parsed_exercice["%s_checkbox" % key] = 1
                    else:
                        parsed_exercice["%s_checkbox" % key] = 0
                    if "eqweight" in variable:
                        parsed_exercice["%s_eqweight" % key] = 1
                    else:
                        parsed_exercice["%s_eqweight" % key] = 0
                else:
                    variable = variable.replace("_ENDLINE_", "\n")
                    # when variable = asis(variable)
                    if variable.startswith("asis("):
                        parsed_exercice[key] = variable[5:-1]
                    else:
                        parsed_exercice[key] = variable
                    # On détecte un eventuel souci dans le modèle (cas ou variable = $$key$$)
                    if parsed_exercice[key] == ("&#36;&#36;%s&#36;&#36;" % key):
                        parsed_exercice[key] = self.getVariablesDefaut(modele)[key]
                        # parsed_exercice[key] = variable.decode("iso-8859-1").encode("utf-8")

            if modele == "qcmsuite":
                i = 0
                pattern = 'textarea="(.*?)"_ENDLINE_'
                m = re.compile(pattern)
                recherche = m.search(fichier)
                variable = recherche.group(1)
                variable = variable.replace("_ENDLINE_", "")
                list_id_questions = variable.split(" ")
                parsed_exercice["list_id_questions"] = list_id_questions
                for id_question in list_id_questions:
                    pattern = "text{%s=(.*?)}_ENDLINE_" % id_question
                    m = re.compile(pattern)
                    recherche = m.search(fichier)
                    variable = recherche.group(1)
                    variable = self.extendResearch(variable, fichier, recherche.end())
                    variable = variable.replace("_ENDLINE_", "\n")

                    # when variable = asis(variable)
                    if variable.startswith("asis("):
                        variable = variable[5:-1]

                    # Si variable commence par "Qtitle", le premier saut de ligne est remplacé par un  "<br/>"
                    if variable.startswith("Qtitle"):
                        variable = variable.replace("\n", "<br/>", 1)
                    list_variable = variable.split("\n")
                    parsed_exercice["enonce%s" % str(i)] = list_variable[0].replace("<br/>", "\n")
                    parsed_exercice["feedback%s" % str(i)] = list_variable[1]
                    parsed_exercice["reponses%s" % str(i)] = "\n".join(list_variable[2:])
                    i = i + 1
            return parsed_exercice

    def extendResearch(self, variable, fichier, end_offset):
        u"""Permet d'etendre une recherche d'expression reguliere dans le cas ou le caractere d'arret aurait été imbriqué dans la variable."""
        # LOG.info("----- extendResearch -----")
        CountLeft = len(re.findall("\{", variable))
        CountRight = len(re.findall("\}", variable))
        if CountLeft > CountRight:
            # Au cas ou la chaine "}_ENDLINE_" avait été introduite dans la variable, le nombre d'accolades sera impair.
            # On continue alors la recherche jusqu'à la prochaine occurence.
            string = fichier[end_offset:]
            m2 = re.search("}_ENDLINE_", string)
            variable = "%s}_ENDLINE_%s" % (variable, string[:m2.start()])
            # Et on recommence en récursif tant que le nombre d'accolades n'est pas pair.
            new_offset = end_offset + len(string[:m2.start()]) + len("}_ENDLINE_")
            return self.extendResearch(variable, fichier, new_offset)
        else:
            return variable

    def getParamListeExos(self, exo_params, mode="autoeval"):
        u"""permet d'obtenir des parametres par défaut pour visualiser un exo en dehors d'une feuille."""
        # LOG.info("----- getParamListeExos -----")
        if exo_params:
            exo = exo_params.replace("*-*", "&")
            return exo
        else:
            if self.getModele() == "groupe":
                exo = "&exo=".join(self.getListeIdsExos())
            else:
                exo = self.getId()

            if mode == "examen":
                intro_check = "1,2,4"
            else:
                intro_check = "1,2,3,4"
            # Apparament, classes/fr n'a plus d'influence sur la langue. c'est le paremetre "lang" qui prime.

            # Valeur du parametre "intro_check" :
            # 1 = Toujours proposer une bonne réponse.
            # 2 = Pénalité pour une mauvaise réponse.
            # 3 = Afficher la bonne réponse. (A EVITER EN EXAMEN)
            # 4 = Permettre les indications (si disponibles).

            # parametre random = [0]: Proposer les exercices de la série dans l'ordre.
            # parametre scoredelay = x : nbre de secondes du chronometre
            # parametre intro_presentsol = [0:jamais, 1:si note non maxi, 2:toujours ] : Donner une solution (si disponible)?

            # qnum représente le nombre d'exercices à effectuer pour avoir une note.
            # qcmlevel représente la sévérité de notation.
            return "exo=%s&qnum=%s&qcmlevel=1&intro_qcmpresent=4&intro_presentsol=1&intro_check=%s&intro_expert=yes&scoredelay=0&module=classes/fr" % (exo, self.getQnum(), intro_check)

    def convertLangToWIMS(self, portal_lang):
        """Permet d'obtenir le code de langue WIMS a partir du code de langue de Plone."""
        # LOG.info("----- convertLangToWIMS -----")
        return jalon_utils.convertLangToWIMS(portal_lang)

    def test(self, condition, valeurVrai, valeurFaux):
        """permet de tester une condition, puis de renvoyer une valeur en fonction."""
        # LOG.info("----- test condition (%s) -----" % condition)
        return jalon_utils.test(condition, valeurVrai, valeurFaux)

    def getUrlServeur(self):
        """fournit l'URL du serveur WIMS."""
        # LOG.info("----- getUrlServeur -----")
        return self.aq_parent.wims("getAttribut", "url_connexion")

    def formaterTitreWIMS(self, titre):
        u"""Renvoit une chaine dépourvye d'un ensemble de caracteres susceptibles de corrompre un exercice."""
        # LOG.info("----- formaterTitreWIMS -----")
        listeReplace_titre_wims = ["<", ">", "{", "}", "(", ")", "[", "]", "$", "&", "?", "!", ",", "\"", "\'", ";", "\\", "/"]
        for lettre in listeReplace_titre_wims:
            titre = titre.replace(lettre, " ")
        # Replace multiple spaces by one (and remove starting&ending spaces)
        return re.sub(' +', ' ', titre.strip())

    def setProperties(self, dico):
        u"""Définit les propriétés d'un jalonexercicewims."""
        # LOG.info("----- setProperties -----")
        for key in dico.keys():
            if key == "Title":
                dico[key] = self.formaterTitreWIMS(dico[key])
            self.__getattribute__("set%s" % key)(dico[key])
        self.reindexObject()

        # Changement du titre de l'exo en cascade dans les autoeval/examens quand on le modifie dans "Mon Espace"
        items = self.getRelatedItems()
        for item in items:
            if item.Type() == "AutoEvaluation-Examen":
                # dico = item.getInfosElement()
                dico = item.getDocumentsProperties()
                if dico:
                    if self.getId() in dico:
                        dico[self.getId()]["titreElement"] = self.Title()
                    item.setDocumentsProperties(dico)
                    item.reindexObject()

    def parser_permalien(self, permalien):
        u"""vérifie qu'un permalien est correct, et renvoit sa version corrigée.

        Cette fonction ne renvoit que les variables de l'URL. Inutile de sauvegarder le DNS et "http://"...

        """
        # LOG.info("----- parser_permalien -----")
        new_permalink = ""
        message = ""
        # On applique un ensemble de filtre pour éviter que des utilisateurs collent des liens mal formés, du style
        # http://ticewims.unice.fr/wims/wims.cgi?module=local/qcmC2i.fr" title="
        # ==> sera remplacé par
        # http://ticewims.unice.fr/wims/wims.cgi?module=local/qcmC2i.fr&cmd=new
        # strip retire les espace avant/apres, split[0] coupe au premier espace, et split('"') ne prend que ce qui se trouve avant un guillemet
        new_permalink = permalien.strip().split()[0].split('"')[0]

        test_string = "http://"
        if new_permalink.count(test_string) > 1:
            # We take only the last URL.
            new_permalink = "%s%s" % (test_string, new_permalink.split(test_string)[-1])
            message = _(u"Un exercice ne peux avoir qu'un seul permalien.<br/>")

        if new_permalink != permalien:
            message = message + _(u"Votre permalien était incorrect, et il a été corrigé automatiquement. Merci de vérifier son bon fonctionnement.")

        url = urlparse(new_permalink)
        params = {}
        for part in url[4].split('&'):
            variable = part.split('=')
            # S'il n'y a aucun "=" dans la chaine, variable ne contiendra qu'un element.
            if (len(variable) > 1):
                if variable[0] not in params.keys():
                    params[variable[0]] = variable[1]
                else:
                    params[variable[0]] = params[variable[0]] + "," + variable[1]
        if "module" in params.keys():
            new_permalink = "module=%s" % params['module']
            for key in sorted(params.iterkeys()):
                if key not in ['cmd', 'session', 'module']:
                    new_permalink = "%s&%s=%s" % (new_permalink, key, params[key])
            if message != "":
                self.plone_utils.addPortalMessage(message, type='warning')
            return new_permalink
        else:
            message = _(u"Permalien incorrect. Assurez-vous de coller un permalien correct svp.")
            self.plone_utils.addPortalMessage(message, type='warning')
            # Dans ce cas, l'exercice ne sera pas modifié.
            return ""

    def addRelatedItem(self, item_a_ajouter):
        u"""Ajoute un objet aux relatedItems du JalonExerciceWims actuel, puis réindexe l'exo."""
        # LOG.info("----- addRelatedItem -----")
        relatedItems = self.getRelatedItems()
        if item_a_ajouter not in relatedItems:
            relatedItems.append(item_a_ajouter)
            self.setRelatedItems(relatedItems)
            self.reindexObject()

    def removeRelatedItem(self, item_a_retirer):
        u"""Retire un objet des relatedItems du JalonExerciceWims actuel, puis réindexe l'exo."""
        # LOG.info("----- removeRelatedItem -----")
        relatedItems = self.getRelatedItems()
        if item_a_retirer in relatedItems:
            relatedItems.remove(item_a_retirer)
            self.setRelatedItems(relatedItems)
            self.reindexObject()

    def updateRelatedItems(self):
        """Update the exercice related items (checks if each related item still exists, and if they are really still connected."""
        # LOG.info("----- updateRelatedItems -----")
        relatedItems = self.getRelatedItems()
        self_id = self.getId()
        deleted = []
        for item in relatedItems:
            # LOG.info("item_type = %s" % item.portal_type)
            if item.portal_type == "JalonCoursWims":
                # Cas de l'activité WIMS
                if self_id not in item.getListeExercices():
                    # LOG.info("L'exercice n'est plus dans l'activité reliée ! On le retire.")
                    deleted.append(item)
                    if self in item.getRelatedItems():
                        item.removeRelatedExercice(self)
                    else:
                        relatedItems.remove(item)
            elif item.portal_type == "JalonExerciceWims":
                # Cas du groupement d'exercice
                if self_id not in item.getListeIdsExos():
                    # LOG.info("L'exercice n'est plus dans le groupe relié ! On le retire.")
                    deleted.append(item)
                    if self in item.getRelatedItems():
                        item.removeRelatedItem(self)
                    relatedItems.remove(item)
        self.setRelatedItems(relatedItems)
        self.reindexObject()
        # Renvoit la liste des corrections effectuées
        return deleted

    def checkRoles(self, user, context, action="view"):
        u"""Permet de vérifier si l'utilisateur courant a le droit d'accéder à un exercice.

        # Si le droit n'est pas accordé, un message est affiché à l'utilisateur.
        # Si on suspecte une tentative de fraude, l'administrateur recoit un message.
        # action indique si l'utilisateur tente d'y accéder en lecture ("view") ou écriture ("edit")

        """
        # Pour le moment, action n'est pas utile. Le même role permet d'acceder en lecture / ecriture.
        # LOG.info("----- checkRoles -----")
        user_roles = user.getRolesInContext(context)
        if 'Owner' in user_roles or 'Manager' in user_roles:
            return True
        else:
            if 'Anonymous' in user_roles:
                # Normalement, le mecanisme de gestion des droits de Plone ne pemretrra de toute facon pas un anonyme d'accéder à un exercice.
                # Mais ceci peux tout de même se produire dans le cas ou un exercice a été rendu public via son insertion dans un cours public justement.
                message = _(u"Vous êtes déconnecté. Merci de vous connecter pour accéder à cette page.")
                mess_type = "info"
            else:
                message = _(u"Vous tentez d'accéder à une page qui ne vous appartient pas. Une suspicion de fraude vous concernant a été envoyée à l'administrateur du site.")
                mess_type = "alert"
                self.aq_parent.wims("verifierRetourWims", {"rep": '{"status":"ERROR", "message":"checkRoles Failed"}',
                                                           "fonction": "jalonexercicewims.py/checkRoles",
                                                           "message": "Suspicion de fraude de l'utilisateur %s" % user.getId(),
                                                           "jalon_request": context.REQUEST
                                                           })

        context.plone_utils.addPortalMessage(message, type=mess_type)
        return False


registerATCT(JalonExerciceWims, PROJECTNAME)



## Controller Python Script "edit_course_item_visibility_script"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=
##title=
##

from Products.CMFPlone import PloneMessageFactory as _

# context = context
context_object = context
form = context.REQUEST.form
redirection = context.absolute_url()

# Interdit la modification de la visibilité aux utilisateurs anonymes.
membership_tool = context.portal_url.getPortalObject().portal_membership
if membership_tool.isAnonymousUser():
    context.plone_utils.addPortalMessage(_(u'Veuillez vous connecter.'), type='error')
    return context.REQUEST.RESPONSE.redirect(redirection)

is_activity = False
object_type = form["item_id"].split("-", 1)[0]
if object_type in ["BoiteDepot", "AutoEvaluation", "Examen"]:
    is_activity = True
    context_object = getattr(context, form["item_id"])

item_date = DateTime() if form.has_key("date-affichage-now") else DateTime(form["datetime"])

is_update_from_title = False
if form["item_property_name"] == "affElement" and form.has_key("item_parent_title_id"):
    is_update_from_title = True
    context_object.editCourseParentTitleVisibility(form["item_parent_title_id"], item_date)

if form.has_key("is_item_title"):
    # Affichage/masquage d'un titre et son contenu
    context_object.editCourseTitleVisibility(form["item_id"], item_date, form["item_property_name"])
else:
    # Affichage/masquage d'un element
    context_object.editCourseItemVisibility(form["item_id"], item_date, form["item_property_name"], is_update_from_title)

# Cas ou on affiche un document d'une activité
if context.meta_type in ["JalonBoiteDepot", "JalonCoursWims"] and not is_activity:
    redirection = "%s?tab=documents" % context_object.absolute_url()

context.REQUEST.RESPONSE.redirect(redirection)



from my_space_view import MySpaceView
from my_files_view import MyFilesView
from my_adobe_connect_recording_view import MyAdobeConnectRecordingView
from my_wims_exercices_view import MyWimsExercicesView
from my_external_resources_view import MyExternalResourcesView
from my_glossary_terms_view import MyGlossaryTermsView
from my_pod_videos_view import MyPodVideosView
from my_courses_view import MyCoursesView
from course_view import CourseView
from deposit_box_view import DepositBoxView
from wims_activity_view import WimsActivityView
from deposit_box_criteria_view import DepositBoxCriteriaView
from deposit_box_evaluation_view import DepositBoxEvaluationView
from course_statistics_view import CourseStatisticsView
from my_students_view import MyStudentsView



## Controller Python Script "download_deposit_zip_script"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=
##title=
##

request = context.REQUEST

zipfile = context.telechargerDepots(request["HTTP_USER_AGENT"])

request.RESPONSE.setHeader('content-type', 'application/zip')
request.RESPONSE.setHeader('content-length', zipfile["length"])
request.RESPONSE.setHeader('Content-Disposition',' attachment; filename=ArchiveDepot%s.zip' % DateTime().Date())

#request.RESPONSE.redirect("%s/cours_boite_view?menu=depots" % context.absolute_url())

return zipfile["data"]


## Controller Python Script "getFullDescription"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=
##title=Activite edit
##

return context.getRawDescription()


# -*- coding: utf-8 -*-
## Controller Python Script "delete_tag_script"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=
##title=Ajout et/ou modification d'un exercice WIMS
##
#context = context

# Suppression de l'étiquette
tag_id = context.REQUEST["tag_id"]
context.deleteTagFolder(tag_id)

context.REQUEST.RESPONSE.redirect(context.absolute_url())



# -*- coding: utf-8 -*-
"""Vue du template "jalon/theme/browser/view_template/jalon_course/course_add_view.pt"."""
from zope.component import getMultiAdapter
from my_space_view import MySpaceView

from logging import getLogger
LOG = getLogger('[CourseAddView]')


class CourseAddView(MySpaceView):
    """Class View du fichier course_add_view.pt."""

    _course_add_dict = {"mes_fichiers":                 {"folder_id":            "Fichiers",
                                                         "macro_file":           "add_course_files_macro",
                                                         "portal_type":          ["File", "Image", "Document"],
                                                         "course_add_list_icon": "fa fa-files-o",
                                                         "is_display_hide":      True,
                                                         "course_add_js":        "setAttachmentCreator()"},
                        "mes_presentations_sonorisees": {"folder_id":            "Sonorisation",
                                                         "macro_file":           "add_course_adobe_connect_recording_macro",
                                                         "portal_type":          ["JalonConnect"],
                                                         "course_add_list_icon": "fa fa-microphone",
                                                         "is_display_hide":      True,
                                                         "course_add_js":        "setAttachmentCreator()"},
                        "mes_ressources_externes":      {"folder_id":            "Externes",
                                                         "macro_file":           "add_course_external_resources_macro",
                                                         "portal_type":          ["JalonRessourceExterne"],
                                                         "course_add_list_icon": "fa fa-external-link",
                                                         "is_display_hide":      True,
                                                         "course_add_js":        "setAttachmentCreator()"},
                        "mes_termes_glossaire":         {"folder_id":            "Glossaire",
                                                         "macro_file":           "add_course_glossary_term_macro",
                                                         "portal_type":          ["JalonTermeGlossaire"],
                                                         "course_add_list_icon": "fa fa-font",
                                                         "is_display_hide":      False,
                                                         "course_add_js":        "setTagFilter(true)"},
                        "mes_webconferences":           {"folder_id":            "Webconference",
                                                         "macro_file":           "add_course_adobe_connect_recording_macro",
                                                         "portal_type":          ["JalonConnect"],
                                                         "course_add_list_icon": "fa fa-headphones",
                                                         "is_display_hide":      True,
                                                         "course_add_js":        "setAttachmentCreator()"},
                        "mes_videos_pod":               {"folder_id":            "Video",
                                                         "macro_file":           "add_course_pod_videos_macro",
                                                         "portal_type":          ["JalonRessourceExterne"],
                                                         "course_add_list_icon": "fa fa-youtube-play",
                                                         "is_display_hide":      True,
                                                         "course_add_js":        "setAttachmentCreator()"},
                        "mes_exercices_wims":           {"folder_id":            "Wims",
                                                         "macro_file":           "add_wims_activity_exercice_macro",
                                                         "portal_type":          ["JalonExerciceWims"],
                                                         "course_add_list_icon": "fa fa-random",
                                                         "is_display_hide":      False,
                                                         "course_add_js":        "setTagFilter(true)"},
                        "course_bibliography":          {"folder_id":            "Externes",
                                                         "macro_file":           "add_course_bibliography_macro",
                                                         "portal_type":          ["JalonRessourceExterne"],
                                                         "course_add_list_icon": "fa fa-external-link",
                                                         "is_display_hide":      False,
                                                         "course_add_js":        "setTagFilter(true)"}}

    def __init__(self, context, request):
        """initialise CourseAddView."""
        # LOG.info("----- Init -----")
        MySpaceView.__init__(self, context, request)
        self.context = context
        self.request = request

    def isAnonymous(self):
        """check if connected user is Anonymous."""
        portal_state = getMultiAdapter((self.context, self.request),
                                       name=u'plone_portal_state')
        return portal_state.anonymous()

    def getCourseAddView(self, user, course_path):
        """Fournit les infos d'ajout d'un element dans le cours."""
        LOG.info("----- getCourseAddView -----")
        portal_state = getMultiAdapter((self.context, self.request), name=u'plone_portal_state')
        portal = portal_state.portal()

        course_add_dict = self._course_add_dict[self.context.getId()]
        LOG.info(self.context.getId())

        folder = getattr(getattr(portal.Members, user.getId()), course_add_dict["folder_id"])
        selected_tags_list = folder.getSelectedTags().split(",")

        tags = self.getTags(folder, selected_tags_list)
        tags_dict = tags["tags_dict"]
        tags_list = tags["tags_list"]

        course_add_list = self.getItemsList(folder, selected_tags_list, {"portal_type": course_add_dict["portal_type"]})

        one_and_selected_tag = self.getOneAndSelectedTag(course_add_list, selected_tags_list, tags_dict)

        nb_display_items = len(course_add_list)
        nb_items = len(folder.objectIds())

        course_path_list = course_path.split("/")
        # LOG.info("***** course_path : %s" % course_path)
        course_object = getattr(getattr(portal.cours, course_path_list[0]), course_path_list[1])

        is_course = True
        course_map_form = ""
        wims_exercice_model_list = ""
        # Cours- = id à partir de 09/2012 ; Cours. = id avant 09/2012
        if course_path_list[-1].startswith("Cours-") or course_path_list[-1].startswith("Cours."):
            if self.context.getId() == "mes_termes_glossaire":
                is_course = False
                course_map = course_object.getGlossaire()
            elif self.context.getId() == "course_bibliography":
                is_course = False
                course_map = course_object.getBibliographie()
            else:
                course_map = course_object.getCourseMapList()
                course_map_form = self.getCourseMapForm(course_path)
            course_add_js = course_add_dict["course_add_js"]
        else:
            is_course = False
            course_object = getattr(course_object, course_path_list[-1])
            if course_add_dict["folder_id"] == "Wims":
                course_map = course_object.getListeExercices()
                wims_exercice_model_list = folder.getModelesWims()
            else:
                course_map = course_object.getDocumentsList()
            course_add_js = "setTagFilter(true)"

        return {"tags_list":            tags_list,
                "is_no_items":          one_and_selected_tag["is_no_items"],
                "is_one_tag":           one_and_selected_tag["is_one_tag"],
                "one_tag":              one_and_selected_tag["one_tag"],
                "is_selected_tags":     one_and_selected_tag["is_selected_tags"],
                "macro_file":           course_add_dict["macro_file"],
                "course_add_list_icon": course_add_dict["course_add_list_icon"],
                "nb_display_items":     nb_display_items,
                "nb_items":             nb_items,
                "course_add_list":      course_add_list,
                "is_course":            is_course,
                "course_map":           course_map,
                "course_map_form":      course_map_form,
                "is_display_hide":      course_add_dict["is_display_hide"],
                "folder_id":            course_add_dict["folder_id"],
                "folder_link":          "%s/mes_ressources/%s" % (portal.absolute_url(), self.context.getId()),
                "course_link":          course_object.absolute_url(),
                "course_add_js":        course_add_js,
                "wims_exercice_model_list": wims_exercice_model_list}

    def getCourseMapForm(self, course_path):
        """redirige vers course_map_form."""
        # LOG.info("----- getCourseMapForm -----")
        return self.context.restrictedTraverse("cours/%s/course_map_form" % course_path)()



# -*- coding: utf-8 -*-
""" Annonces Jalon. """
from zope.interface import implements
from Products.Archetypes.public import *
from Products.ATContentTypes.content.document import ATDocumentBase, ATDocumentSchema
from Products.ATContentTypes.content.base import registerATCT
from Products.CMFCore.utils import getToolByName

from jalon.content import contentMessageFactory as _
from jalon.content.config import PROJECTNAME
from jalon.content.interfaces import IJalonAnnonce

import jalon_utils

JalonAnnonceSchema = ATDocumentSchema.copy() + Schema((
    LinesField("publics",
               required=False,
               accessor="getPublics",
               searchable=False,
               widget=LinesWidget(label=_(u"Publics cible d'une annonce"),
                                  visible={'view': 'visible', 'edit': 'invisible'})
               ),
))


class JalonAnnonce(ATDocumentBase):

    """ Une annonce dans un cours. """

    implements(IJalonAnnonce)
    meta_type = 'JalonAnnonce'
    schema = JalonAnnonceSchema
    schema['description'].required = True
    schema['description'].widget.label = "Description"
    schema['text'].required = False
    schema['text'].mode = "r"

    def setProperties(self, dico):
        for key in dico.keys():
            self.__getattribute__("set%s" % key)(dico[key])
        self.reindexObject()

    def getDescriptionAff(self):
        return self.Description().replace("\n", "<br/>")

    def getLocaleDate(self):
        return jalon_utils.getLocaleDate(self.modified())

    def test(self, condition, valeurVrai, valeurFaux):
        return jalon_utils.test(condition, valeurVrai, valeurFaux)

    def envoyerAnnonce(self):
        parent = self.aq_parent.aq_parent
        portal = self.portal_url.getPortalObject()
        portal_membership = getToolByName(portal, 'portal_membership')
        jalon_properties = getToolByName(portal, 'portal_jalon_properties')
        authMember = portal_membership.getAuthenticatedMember()
        de = authMember.getProperty("email")
        fullname = authMember.getProperty("fullname")
        if not fullname:
            fullname = authMember.getProperty("displayName")
        dico = {"de": de,
                "a": "",
                "objet": self.Title(),
                "message": self.Description(),
                "cours": parent.Title(),
                "auteur": fullname}
        diffusion = {"activer_diffusion": jalon_properties.getJalonProperty("activer_liste_diffusion"),
                     "type_diffusion": jalon_properties.getJalonProperty("type_diffusion").split(",")}
        for public in self.getPublics():
            typeELP, codeELP = public.split("*-*")
            if typeELP == "invitationsemail":
                for invitation in parent.getInvitations():
                    dico["a"] = invitation
                    jalon_utils.envoyerMail(dico)
            listeEtudiants = []
            if diffusion["activer_diffusion"] and typeELP in diffusion["type_diffusion"]:
                dico["a"] = diffusion["format_diffusion"].replace("*-*code*-*", codeELP)
                jalon_utils.envoyerMail(dico)
            elif typeELP in ["etape", "ue", "uel", "groupe"]:
                listeEtudiants = parent.aq_parent.getListeEtudiants(codeELP, typeELP)
                for etudiant in listeEtudiants:
                    if etudiant["EMAIL_ETU"]:
                        dico["a"] = etudiant["EMAIL_ETU"]
                        jalon_utils.envoyerMail(dico)
            else:
                if typeELP == "groupeperso":
                    listeEtudiants = parent.getNominativeRegistration()
                if typeELP == "coauteurs":
                    listeEtudiants = parent.getCoAuteursCours()
                if typeELP == "colecteurs":
                    listeEtudiants = parent.getCourseReader()
                if listeEtudiants:
                    for etudiant in listeEtudiants:
                        if etudiant["email"]:
                            dico["a"] = etudiant["email"]
                            jalon_utils.envoyerMail(dico)
        jalon_utils.envoyerMail({"de":      de,
                                 "a":       de,
                                 "objet":   self.Title(),
                                 "message": self.Description(),
                                 "cours":   parent.Title(),
                                 "auteur":  fullname})
        cours = self.aq_parent.aq_parent
        auteurCours = cours.getAuteurPrincipal()
        if not auteurCours:
            auteurCours = cours.Creator()
        if self.Creator() != auteurCours:
            infosAuteurCours = jalon_utils.getIndividu(auteurCours, "dict")
            jalon_utils.envoyerMail({"de":      de,
                                     "a":       infosAuteurCours["email"],
                                     "objet":   self.Title(),
                                     "message": self.Description(),
                                     "cours":   parent.Title(),
                                     "auteur":  fullname})

registerATCT(JalonAnnonce, PROJECTNAME)



## Controller Python Script "edit_course_announce_script"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=
##title=
##

form = context.REQUEST.form

announce_publics = tuple(form["publicsElement"]) if form.has_key("publicsElement") else ()
mail_announce = form["mailAnnonce"] if form.has_key("mailAnnonce") else False

context.setProperties({"Title":       form["title"],
                       "Description": form["description"],
                       "Publics":     announce_publics})

if mail_announce:
    context.envoyerAnnonce()

context.REQUEST.RESPONSE.redirect(context.aq_parent.aq_parent.absolute_url())



## Controller Python Script "create_external_bu_script"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=
##title=
##

REQUEST = context.REQUEST
form = REQUEST.form
user_id = form["member_id"]
external_resources_folder = context.getMySubSpaceFolder(user_id, "Externes")

for recordid in form["recordsid"]:
    object_id = "Externe-%s-%s" % (user_id, recordid)
    object_exist = getattr(external_resources_folder, object_id, None)
    if not object_exist:
        external_resources_folder.invokeFactory(type_name='JalonRessourceExterne', id=object_id)
        object_new = getattr(external_resources_folder, object_id)
        object_new.editFromBUCatalogBU(recordid)

context.REQUEST.RESPONSE.redirect(context.absolute_url())



# -*- coding: utf-8 -*-
# Controller Python Script "add_tag_object_script"
# bind container=container
# bind context=context
# bind namespace=
# bind script=script
# bind subpath=traverse_subpath
# parameters=
# title=
##

try:
    tags_list = tuple(context.REQUEST.form["listeTag"])
except:
    tags_list = ()
context.setSubject(tags_list)
context.reindexObject()

if context.REQUEST.HTTP_X_REQUESTED_WITH != 'XMLHttpRequest':
    context.REQUEST.RESPONSE.redirect(context.aq_parent.absolute_url())
else:
    return context.restrictedTraverse(context.aq_parent.getMySpaceFolder())()



## Controller Python Script "mark_out_course_map_item_script"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=
##title=
##

form = context.REQUEST.form

context.setCourseProperties({"AvancementPlan": [form["mark_out_item_id"], form["mark_out_item_text"]]})

context.REQUEST.RESPONSE.redirect(context.absolute_url())



## Controller Python Script "edit_deposit_tab_script"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=
##title=
##

form = context.REQUEST.form

if int(form["CorrectionIndividuelle"]) == 0:
    NotificationCorrection = 0
else:
    NotificationCorrection = int(form["NotificationCorrection"])
if int(form["Notation"]) == 0:
    NotificationNotation = 0
else:
    NotificationNotation = int(form["NotificationNotation"])
dico = {"CorrectionIndividuelle": int(form["CorrectionIndividuelle"]),
        "NotificationCorrection": NotificationCorrection,
        "Notation":               int(form["Notation"]),
        "NotificationNotation":   NotificationNotation,
        "AccesDepots":            int(form["AccesDepots"])}

context.setProperties(dico)

context.REQUEST.RESPONSE.redirect(context.absolute_url())



# -*- coding: utf-8 -*-
"""WimsActivityView python Class."""
# from Products.Five.browser import BrowserView
# from zope.component import getMultiAdapter
from course_view import CourseView
from jalon.content import contentMessageFactory as _

from logging import getLogger
LOG = getLogger('[WimsActivityView]')


class WimsActivityView(CourseView):
    """Wims Activity View Class."""

    """def __init__(self, context, request):
        # initialize WimsActivityView.
        # LOG.info("----- Init -----")
        CourseView.__init__(self, context, request)
    """

    def getBreadcrumbs(self):
        """Get current page breadcrumbs."""
        # LOG.info("----- getBreadcrumbs -----")
        portal = self.context.portal_url.getPortalObject()
        parent = self.context.aq_parent
        self_link = self.context.absolute_url()
        response = [{"title": _(u"Mes cours"),
                     "icon":  "fa fa-university",
                     "link":  "%s/mes_cours" % portal.absolute_url()},
                    {"title": parent.Title(),
                     "icon":  "fa fa-book",
                     "link":  parent.absolute_url()},
                    {"title": self.context.Title(),
                     "icon":  self.context.getIconClass(),
                     "link":  self_link}]
        if self.request.ACTUAL_URL.split("/")[-1] == "wims_activity_exercice_view":
            response.append({"title": _("Exercice(s)"), "icon":  "fa fa-random", "link": "%s/wims_activity_exercice_view" % self_link})
        return response

    def getWimsActivityView(self, user, mode_etudiant, tab, is_ajax):
        """Get Wims Activity View."""
        # LOG.info("----- getWimsActivityView (Start) tab='%s' -----" % tab)
        user_id = user.getId()
        my_view = {"is_anonymous": self.isAnonymous()}

        if tab not in ["exercices", "documents", "results"]:
            tab = "exercices"
        my_view["tab"] = tab

        my_wims_activity = self.context
        portal = my_wims_activity.portal_url.getPortalObject()
        portal_link = portal.absolute_url()
        nb_exos = my_wims_activity.getNbExercices()
        instruction_text = my_wims_activity.Description()
        if instruction_text:
            my_view["instruction_class"] = "panel callout radius"
            my_view["instruction_text"] = instruction_text
        else:
            my_view["instruction_class"] = "panel radius"
            my_view["instruction_text"] = "Consigne non renseignée"

        my_view["activity_id"] = my_wims_activity.getId()
        my_view["activity_link"] = my_wims_activity.absolute_url()
        my_view["activity_type"] = my_wims_activity.getTypeWims()

        if my_view["activity_type"] == "Examen":
            my_view["activity_link_title"] = "Accéder à cet examen WIMS"
        else:
            my_view["activity_link_title"] = "Accéder à cet entrainement WIMS"

        my_view["activity_icon"] = my_wims_activity.getIconClass()
        my_view["duree"] = my_wims_activity.getDuree()

        my_view["is_personnel"] = my_wims_activity.isPersonnel(user, mode_etudiant)

        if (not mode_etudiant) and my_view["is_personnel"]:
            my_view["mode_etudiant"] = "false"
        else:
            my_view["mode_etudiant"] = mode_etudiant

        if is_ajax or my_view["is_anonymous"]:
            my_view["came_from"] = "%s/acl_users/credentials_cookie_auth/require_login?came_from=%s" % (portal_link, my_wims_activity.jalon_quote(my_view["activity_link"]))
            # my_view["came_from"] = "%s/login_form?came_from=%s" % (my_view["activity_link"], my_wims_activity.jalon_quote(my_view["activity_link"]))
            # LOG.info("----- getWimsActivityView (Early Ended : ajax or anonymous) -----")
            return my_view

        # my_view["wims_activity_instruction"] = {"href":  "%s/edit_wims_activity_instruction_form?tab=%s" % (my_view["activity_link"], tab),
        #                                        "icon":  "fa-pencil",
        #                                        "text":  "Modifier"}

        my_view["wims_activity_visibility"] = my_wims_activity.isAfficherElement(my_wims_activity.getDateAff(), my_wims_activity.getDateMasq())

        my_view["wims_activity_edit"] = []
        if my_view["wims_activity_visibility"]["val"]:
            my_view["wims_activity_edit"].append({"href": "%s/edit_course_item_visibility_form?item_id=%s&tab=%s" % (my_view["activity_link"], my_view["activity_id"], tab),
                                                  "icon": "fa-eye-slash",
                                                  "text": "Masquer"})
        else:
            my_view["wims_activity_edit"].append({"href": "%s/edit_course_item_visibility_form?item_id=%s&tab=%s" % (my_view["activity_link"], my_view["activity_id"], tab),
                                                  "icon": "fa-eye",
                                                  "text": "Afficher"})
            if my_view["activity_type"] == "Examen" and nb_exos > 0:
                my_view["wims_activity_edit"].append({"href": "%s/wims_activity_exercice_view" % my_view["activity_link"],
                                                      "icon": "fa-laptop",
                                                      "text": "Tester les exercices"})
        my_view["wims_activity_edit"].append({"href": "%s/edit_wims_activity_form?tab=%s" % (my_view["activity_link"], tab),
                                              "icon": "fa-pencil",
                                              "text": "Modifier"})

        my_view["is_personnel_or_wims_activity_visible"] = True if my_view["is_personnel"] or my_view["wims_activity_visibility"]['val'] != 0 else False
        my_view["is_student_and_wims_activity_hidden"] = True if (not my_view["is_personnel"]) and my_view["wims_activity_visibility"]['val'] == 0 else False
        my_view["is_display_mod"] = my_wims_activity.isAuteurs(user_id)

        my_view["wims_activity_tabs"] = []

        my_view["wims_activity_tabs"].append({"href":      "%s?tab=exercices&mode_etudiant=%s" % (my_view["activity_link"], mode_etudiant),
                                              "css_class": " selected" if tab == "exercices" else "",
                                              "icon":      "fa-random",
                                              "text":      "Exercices",
                                              "nb":        my_wims_activity.getNbExercices()})
        activity_path = my_wims_activity.getPhysicalPath()
        activity_path = "/".join([activity_path[-3], activity_path[-2], activity_path[-1]])

        # Boutons d'ajouts d'exercices, affichés uniquement pour le créateur d'une activité masquée et qui n'est pas un examen déjà activé
        if (tab == "exercices" and
                my_view['is_personnel'] and
                not my_view['wims_activity_visibility']['val'] and
                not my_wims_activity.getIdExam() and
                my_wims_activity.getCreateur() == user_id):
            my_view["id_groupement"] = my_wims_activity.getGroupement()
            my_view["exercices_add"] = self.getExerciceAdderMenuList(portal_link, activity_path)
            # my_view["exercices_list"] = my_wims_activity.displayDocumentsList(my_view["is_personnel"], portal)

        my_view["wims_activity_tabs"].append({"href":      "%s?tab=documents&mode_etudiant=%s" % (my_view["activity_link"], mode_etudiant),
                                              "css_class": " selected" if tab == "documents" else "",
                                              "icon":      "fa-upload",
                                              "text":      "Documents enseignants",
                                              "nb":        my_wims_activity.getNbSujets(my_view["is_personnel"])})
        if tab == "documents":
            my_view["documents_add"] = self.getCourseItemAdderMenuList(my_view["activity_link"], activity_path, portal)["my_space"]
            my_view["documents_list"] = my_wims_activity.displayDocumentsList(my_view["is_personnel"], portal)

        # my_view["is_resultats_tab"] = True if tab == "results" else False
        if my_view["is_personnel"]:
            my_view["wims_activity_tabs"].append({"href":      "%s?tab=results&mode_etudiant=%s" % (my_view["activity_link"], mode_etudiant),
                                                  "css_class": " selected" if tab == "results" else "",
                                                  "icon":      "fa-trophy",
                                                  "text":      "Résultats"})

        # LOG.info("----- getWimsActivityView (End) -----")
        return my_view

    def getWimsSession(self, user, isCoAuteur, isAnonymous):
        """Obtain a new or existing session from WIMS, and returns wims params."""
        if isAnonymous:
            message = _(u"Désolé, seul un utilisateur connecté peux participer aux activités d'un cours.")
            # message = _(u"Le coefficient de l'exercice '${item_title}' a bien été modifié.",
            #            mapping={'item_title': param["title"]})
            self.context.plone_utils.addPortalMessage(message, type='warning')
            return None
        elif isCoAuteur:
            user_id = 'supervisor'
        elif not isAnonymous:
            user_id = user.getId()
        return self.context.authUser(user_id, self.context.getClasse(), self.request, session_keep=True)

    def getExerciceAdderMenuList(self, portal_link, activity_path):
        """Fournit la liste des liens permettant d'ajouter des exercices à une activité WIMS."""
        # portal_link = portal.absolute_url()
        # portal_link = self.portal_url
        item_adder_list = [{"item_link":  "%s/mes_ressources/mes_exercices_wims/course_add_view?course_path=%s" % (portal_link, activity_path),
                                "item_title": "Attacher un exercice WIMS de Mes ressources",
                                "item_icon":  "fa fa-fw fa-random",
                                "item_name":  "Exercice WIMS"},
                           {"item_link":  "%s/cours/%s/import_wims_activity_form?type=hotpot" % (portal_link, activity_path),
                                "item_title": "Importer des exercices",
                                "item_icon":  "fa fa-fw fa-level-down",
                                "item_name":  "Import HotPotatoes"}]

        return item_adder_list



# -*- coding: utf-8 -*-
##Script (Python) "edit_course_author_script"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=
##title=
##

context.setAuteur(context.REQUEST.form)
context.REQUEST.RESPONSE.redirect(context.absolute_url())



## Controller Python Script "modify_archive"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=
##title=
##

user_id = context.REQUEST.form["user_id"]
context.modifyArchive(user_id)

context.REQUEST.RESPONSE.redirect("%s/cours/%s?onglet=2" % (context.portal_url.getPortalObject().absolute_url(), user_id))



## Script (Python) "getConnexionRequire"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=
##title=CAS Logout
##

from Products.CMFCore.utils import getToolByName

portal = context.portal_url.getPortalObject()
portal_jalon_properties = getToolByName(context, 'portal_jalon_properties')

return {"site": portal.Title(),
        "maintenance":              portal_jalon_properties.getPropertiesMaintenance(),
        "activer_message_general":  portal_jalon_properties.getJalonProperty("activer_message_general"),
        "message_general":          portal_jalon_properties.getJalonProperty("message_general"),
        "activer_lien_sesame":      portal_jalon_properties.getJalonProperty("activer_lien_sesame"),
        "lien_sesame":              portal_jalon_properties.getJalonProperty("lien_sesame"),
        "lien_bug":                 portal_jalon_properties.getJalonProperty("lien_assitance"),
        "etablissement":            portal_jalon_properties.getJalonProperty("etablissement"),
        "activer_cas":              portal_jalon_properties.getJalonProperty("activer_cas"),
        "activer_creationcompte":   portal_jalon_properties.getJalonProperty("activer_creationcompte"),
       }


## Controller Python Script "edit_course_map_item_script"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=
##title=Edite un élément du plan interactif
##
from DateTime import DateTime

now = DateTime()
# context = context
form = context.REQUEST.form

if form["item_type"] == "Titre":
    # Supp. marquage HTML éventuellement saisi.
    form["title"] = context.supprimerMarquageHTML(form["title"])

if not form.has_key("item_id"):
    item_id = "%s-%s-%s" % (form["item_type"], form["user_id"], ''.join([now.strftime('%Y%m%d'), now.strftime('%H%M%S')]))
    context.addItemInCourseMap(item_id, form["map_position"])
    context.addItemProperty(item_id, form["item_type"], form["title"], form["user_id"], "", None)
else:
    item_display_in_course_map = True if form.has_key("display_in_plan") else False
    context.editCourseMapItem(form["item_id"], form["title"], item_display_in_course_map)
    if form["typeElement"] in ["BoiteDepot", "AutoEvaluation", "Examen"]:
        activite = getattr(context, form["item_id"])
        activite.setProperties({"Title": form["title"]})
context.setCourseProperties({"DateDerniereModif": DateTime()})

return "%s/display_course_map_page" % context.absolute_url()



## Controller Python Script "activate_webconference_script"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=
##title=
##

# context = context
request = context.REQUEST
form = request.form

context.addItemInCourseMap(form["meeting_id"], "fin_racine")
context.addItemProperty(form["meeting_id"], "SalleVirtuelle", form["meeting_title"], form["meeting_owner"], "", None)

request.RESPONSE.redirect(context.absolute_url())
#return "%s/display_course_map_page" % context.absolute_url()



## Controller Python Script "add_course_external_resource_script"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=
##title=Ajoute une activité au cours
##

form = context.REQUEST.form

if form["title"] in ["", " ", None] or form["lien"] in ["", " ", None]:
    if "biblio" in form:
        redirection = "%s/create_and_add_course_external_resource_form?biblio=true" % context.absolute_url()
    else:
        redirection = "%s/create_and_add_course_external_resource_form" % context.absolute_url()
else:
    user_id = form["user_id"]
    portal = context.portal_url.getPortalObject()
    portal_workflow = portal.portal_workflow
    folder_object = context.getMySubSpaceFolder(user_id, "Externes", portal)

    external_id = "Externe-%s-%s" % (user_id, DateTime().strftime("%Y%m%d%H%M%S"))
    folder_object.invokeFactory(type_name="JalonRessourceExterne", id=external_id)
    external_object = getattr(folder_object, external_id)

    properties = {"Title":       form["title"],
                  "Description": form["description"]}

    if "iframe" in form["lien"] or "embed" in form["lien"]:
        external_type = "Lecteur exportable"
        properties["Lecteur"] = form["lien"]
    else:
        external_type = "Lien web"
        properties["Urlbiblio"] = form["lien"]
    properties["TypeRessourceExterne"] = external_type

    external_object.setProperties(properties)

    if "biblio" in form:
        context.addMySpaceItemBibliography(folder_object, external_id, external_type, user_id)
    else:
        display_item = DateTime() if "display_item" in form and form["display_item"] == "1" else ""
        display_in_plan = True if "display_in_plan" in form and form["display_in_plan"] == "1" else False
        map_items_position = form["map_position"] if form.has_key("map_position") else None

        context.addMySpaceItem(folder_object, external_id, external_type, user_id, display_item, map_items_position, display_in_plan, portal_workflow)

    if context.getId().startswith("Cours-"):
        redirection = context.absolute_url()
    else:
        redirection = "%s?tab=documents" % context.absolute_url()
    #context.REQUEST.RESPONSE.redirect(context.absolute_url())
    #return "%s/display_course_map_page" % context.absolute_url()

context.REQUEST.RESPONSE.redirect(redirection)


## Controller Python Script "create_glossary_term_script"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=
##title=
##

REQUEST = context.REQUEST
form = REQUEST.form

user_id = form["member_id"]
glossary_terms_folder = context.getMySubSpaceFolder(user_id, "Glossaire")
object_id = glossary_terms_folder.invokeFactory(type_name='JalonTermeGlossaire', id="Glossaire-%s-%s" % (user_id, DateTime().strftime("%Y%m%d%H%M%S")))

object_new = getattr(glossary_terms_folder, object_id)

properties_dict = {"Title":                form["title"],
                   "Description":          form["description"]}

object_new.setProperties(properties_dict)

context.REQUEST.RESPONSE.redirect(context.absolute_url())



# -*- coding: utf-8 -*-

from zope.interface import implements
from Products.Archetypes import public as atpublic
from Products.ATContentTypes.content.document import ATDocumentBase, ATDocumentSchema
from Products.ATContentTypes.content.base import registerATCT

from jalon.content import contentMessageFactory as _
from jalon.content.config import PROJECTNAME
from jalon.content.interfaces import IJalonConnect

import copy
import jalon_utils

JalonConnectSchema = ATDocumentSchema.copy() + atpublic.Schema((
    atpublic.StringField(
        'dateAjout',
        required = False,
        accessor = 'getDateAjout',
        searchable = False,
        widget = atpublic.StringWidget(
            label = _(u"Date de création sur Adobe Connect")
            )),
    atpublic.StringField(
        "duree",
        required = False,
        accessor = "getDuree",
        searchable = False,
        widget = atpublic.StringWidget(
            label = _(u"Durée"),
            )),
    atpublic.StringField(
        "dateUS",
        required = False,
        accessor = "getDateUS",
        searchable = False,
        widget = atpublic.StringWidget(
            label = _(u"La date US"),
            )),
    atpublic.StringField(
        "urlEnr",
        required = False,
        accessor = "getUrlEnr",
        searchable = False,
        widget = atpublic.StringWidget(
            label = _(u"URL de l'enregistrement Connect"),
            )),
    ))


class JalonConnect(ATDocumentBase):
    """ Un enregistrement Adobe Connect
    """

    implements(IJalonConnect)
    meta_type = 'JalonConnect'
    schema = JalonConnectSchema
    schema['description'].required = False
    schema['description'].mode = "r"
    schema['text'].required = False
    schema['text'].mode = "r"

    def ajouterTag(self, tag):
        return jalon_utils.setTag(self, tag)

    def setProperties(self, dico):
        for key in dico.keys():
            self.__getattribute__("set%s" % key)(dico[key])
        self.reindexObject()
        items = self.getRelatedItems()
        for item in items:
            if item.portal_type in ["JalonCours"]:
                #modification du titre dans le cours
                element_cours = copy.deepcopy(item.getCourseItemProperties())
                if self.getId() in element_cours:
                    element_cours[self.getId()]["titreElement"] = self.Title()
                    item.setCourseItemsProperties(element_cours)
            if item.portal_type in ["JalonBoiteDepot", "JalonCoursWims"]:
                #modification du titre dans les boite de depots
                dico = copy.deepcopy(item.getDocumentsProperties())
                if self.getId() in dico:
                    dico[self.getId()]["titreElement"] = self.Title()
                    #item.setInfos_element(dico)
                    item.setDocumentsProperties(dico)

registerATCT(JalonConnect, PROJECTNAME)



# -*- coding: utf-8 -*-
from zope.component import getMultiAdapter
from my_space_view import MySpaceView

from jalon.content import contentMessageFactory as _

from logging import getLogger
LOG = getLogger('[MyPodVideosView]')


class MyPodVideosView(MySpaceView):
    """ Class View du fichier my_pod_videos_view.pt
    """

    def __init__(self, context, request):
        # LOG.info("----- Init -----")
        MySpaceView.__init__(self, context, request)
        self.context = context
        self.request = request

    def getBreadcrumbs(self):
        return [{"title": _(u"Mes ressources"),
                 "icon":  "fa fa-folder-open",
                 "link":  self.context.aq_parent.absolute_url()},
                {"title": _(u"Mes contenus UNSPod"),
                 "icon":  "fa fa-youtube-play",
                 "link":  self.context.absolute_url()}]

    def getMyPodVideosView(self, user):
        # LOG.info("----- getMyPodVideosView -----")
        portal_state = getMultiAdapter((self.context, self.request), name=u'plone_portal_state')
        portal = portal_state.portal()

        member_id = user.getId()
        folder = getattr(portal.Members, member_id).Video

        selected_tags_list = folder.getSelectedTags().split(",")

        tags = self.getTags(folder, selected_tags_list)
        tags_dict = tags["tags_dict"]
        tags_list = tags["tags_list"]

        my_pod_videos_list = self.getMyPodVideosList(folder, selected_tags_list, member_id, portal)

        one_and_selected_tag = self.getOneAndSelectedTag(my_pod_videos_list, selected_tags_list, tags_dict)

        nb_display_items = len(my_pod_videos_list)
        nb_items = len(folder.objectIds())

        return {"tags_list":        tags_list,
                "is_no_items":      one_and_selected_tag["is_no_items"],
                "is_one_tag":       one_and_selected_tag["is_one_tag"],
                "one_tag":          one_and_selected_tag["one_tag"],
                "is_selected_tags": one_and_selected_tag["is_selected_tags"],
                "my_items_list":    my_pod_videos_list,
                "nb_display_items": nb_display_items,
                "nb_items":         nb_items,
                "folder_path":      "/".join(folder.getPhysicalPath()),
                "folder_link":      folder.absolute_url()}

    def getMyPodVideosList(self, folder, selected_tags_list, member_id, portal):
        if selected_tags_list == ["last"]:
            self.updateJalonVideos(folder, portal, member_id)
        content_filter = {"portal_type": ["JalonRessourceExterne"]}
        return self.getItemsList(folder, selected_tags_list, content_filter)

    def updateJalonVideos(self, folder, portal, member_id):
        # LOG.info("----- updateJalonVideos -----")
        my_videos_ids = []
        for video in folder.objectIds():
            if member_id in video:
                my_videos_ids.append(video)
        jalon_videos_id = set([int(object_id.split("-")[-1]) for object_id in my_videos_ids])

        portal_elasticsearch = getattr(portal, "portal_jalon_elasticsearch", None)
        response_elasticsearch = portal_elasticsearch.searchElasticsearch("mes_videos", "", 1)

        nb_pages = response_elasticsearch["nb_pages"]
        if not nb_pages:
            videos_del = jalon_videos_id
        else:
            dico_videos_pod = {}
            videos_ids = []
            for video in response_elasticsearch["liste_videos"]:
                videos_ids.append(video["id"])
                dico_videos_pod[video["id"]] = video

            if nb_pages > 1:
                for page in range(2, nb_pages):
                    response_elasticsearch = portal_elasticsearch.searchElasticsearch("mes_videos", "", page)
                    for video in response_elasticsearch["liste_videos"]:
                        videos_ids.append(video["id"])
                        dico_videos_pod[video["id"]] = video

            videos_ids = set(videos_ids)
            videos_del = jalon_videos_id.difference(videos_ids)

            # LOG.info("jalon_videos_id : %s" % jalon_videos_id)
            # LOG.info("videos_ids : %s" % videos_ids)
            # LOG.info("videos_del : %s" % videos_del)

            videos_add = videos_ids.difference(jalon_videos_id)
            # LOG.info(videos_add)
            # LOG.info(dico_videos_pod)
            for video_id in videos_add:
                video = dico_videos_pod[video_id]
                object_id = "Externe-%s-%s" % (member_id, video_id)
                object_video = getattr(folder, object_id, None)
                if not object_video:
                    folder.invokeFactory(type_name='JalonRessourceExterne', id=object_id)
                    object_video = getattr(folder, object_id)
                    video = portal_elasticsearch.searchElasticsearch(type_search="video", term_search=video_id)
                    param = {"Title":                video["title"],
                             "TypeRessourceExterne": "Video",
                             "Videourl":             video["full_url"],
                             "Description":          video["text"],
                             "Lecteur":              video["iframe"],
                             "Videoauteur":          video["owner"],
                             "Videoauteurname":      video["owner_full_name"],
                             "Videothumbnail":       video["thumbnail"]}
                    object_video.setProperties(param)

        for video_id in videos_del:
            # LOG.info("Del Video : %s" % video_id)
            object_id = "Externe-%s-%s" % (member_id, video_id)
            video_object = getattr(folder, object_id)
            relatedItems = ["/".join(related_item.getPhysicalPath()) for related_item in video_object.getRelatedItems()]
            # LOG.info("relatedItems : %s" % str(relatedItems))
            brains = folder.portal_catalog(path={"query": relatedItems, "depth": 0})
            for brain in brains:
                brain_object = brain.getObject()
                # LOG.info("brain_object : %s" % brain_object.getId())
                # LOG.info("brain_object : %s" % brain_object.portal_type)
                if brain_object.portal_type == "JalonCours":
                    brain_object.deleteCourseMapItem(object_id, None)
                else:
                    brain_object.retirerElement(object_id, "sujets")
            folder.manage_delObjects(object_id)



## Controller Python Script "edit_wims_activity_script"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=
##title=Activity WIMS edit script
##

# context = context
form = context.REQUEST.form

# dico = {}

# Cas de l'examen
if "duree" in form:
    dico = {"Title":       form["title"],
            "Description": form["description"],
            "Note_max":    form["note_max"],
            "Duree":       form["duree"],
            "Attempts":    form["attempts"],
            "Verrou":      form["verrou"],
            "Wims_lang":   form["wims_lang"]}

# Cas de l'autoevaluation
elif "note_max" in form:
    dico = {"Title":       form["title"],
            "Description": form["description"],
            "Note_max":    form["note_max"],
            "Wims_lang":   form["wims_lang"]}

# context.setAttributActivite(dico)
context.setProperties(dico)

# Met à jour le titre dans le plan du cours, et la date de dernière modif du cours.
if "Title" in dico:
    context.aq_parent.editCourseMapItem(context.getId(), dico["Title"], False)
    context.aq_parent.setCourseProperties({"DateDerniereModif": DateTime()})

# tab n'existe pas si on modifie l'activite depuis le plan du cours
if "tab" in form:
    redirection = "%s?tab=%s" % (context.absolute_url(), form["tab"])
else:
    redirection = context.aq_parent.absolute_url()

if context.REQUEST.HTTP_X_REQUESTED_WITH != 'XMLHttpRequest':
    context.REQUEST.RESPONSE.redirect(redirection)
else:
    return redirection



## Controller Python Script "add_course_file_script"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=
##title=Ajoute une activité au cours
##

form = context.REQUEST.form

if not "file_file" in form or form["file_file"].filename in ["", " ", None]:
    redirection = "%s/create_and_add_course_file_form" % context.absolute_url()
else:
    user_id = form["user_id"]
    portal = context.portal_url.getPortalObject()
    portal_workflow = portal.portal_workflow
    folder_object = context.getMySubSpaceFolder(user_id, "Fichiers", portal)

    file_name = context.get_id_from_filename(form["file_file"].filename, context)
    file_extension = file_name.split(".")[-1]
    file_type = "Image" if file_extension in ["png", "jpg", "jpeg", "bmp", "svg", "gif"] else "File"

    file_object = getattr(folder_object, file_name, None)
    if not file_object:
        folder_object.invokeFactory(type_name=file_type, id=file_name)
        file_object = getattr(folder_object, file_name)

    title_file = form["title"] if form["title"] else file_name.rsplit('.', 1)[0]
    file_object.setTitle(title_file)
    file_object.setDescription(form["description"])

    if file_type == "Image":
        file_object.setImage(form["file_file"])
    else:
        file_object.setFile(form["file_file"])

    display_item = DateTime() if "display_item" in form and form["display_item"] == "1" else ""
    display_in_plan = True if "display_in_plan" in form and form["display_in_plan"] == "1" else False
    map_items_position = form["map_position"] if form.has_key("map_position") else None

    context.addMySpaceItem(folder_object, file_name, file_type, user_id, display_item, map_items_position, display_in_plan, portal_workflow)

    if context.getId().startswith("Cours-"):
        redirection = context.absolute_url()
    else:
        redirection = "%s?tab=documents" % context.absolute_url()
context.REQUEST.RESPONSE.redirect(redirection)
#return "%s/display_course_map_page" % context.absolute_url()



##Python Script "delete_course_coreader_registration_script"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=
##title=
##

form = context.REQUEST.form

course_reader_list = []
if "auteur-actu" in form:
    course_reader_list = form["auteur-actu"]
context.deleteCourseReader(course_reader_list)

HTTP_X_REQUESTED_WITH = False
if context.REQUEST.HTTP_X_REQUESTED_WITH == 'XMLHttpRequest':
    HTTP_X_REQUESTED_WITH = True

if not HTTP_X_REQUESTED_WITH:
    context.REQUEST.RESPONSE.redirect(context.absolute_url())
else:
    return context.absolute_url()



# -*- coding: utf-8 -*-
from Products.Five.browser import BrowserView
from zope.component import getMultiAdapter

from jalon.content import contentMessageFactory as _
from DateTime import DateTime

from logging import getLogger
LOG = getLogger('[DepositBoxCriteriaView]')


class DepositBoxCriteriaView(BrowserView):
    """Class pour le first_page
    """

    def __init__(self, context, request):
        # LOG.info("----- Init -----")
        BrowserView.__init__(self, context, request)
        self.context = context
        self.request = request

    def isAnonymous(self):
        portal_state = getMultiAdapter((self.context, self.request),
                                       name=u'plone_portal_state')
        return portal_state.anonymous()

    def getBreadcrumbs(self):
        portal = self.context.portal_url.getPortalObject()
        parent = self.context.aq_parent
        deposit_box_link = self.context.absolute_url()
        return [{"title": _(u"Mes cours"),
                 "icon":  "fa fa-university",
                 "link":  "%s/mes_cours" % portal.absolute_url()},
                {"title": parent.Title(),
                 "icon":  "fa fa-book",
                 "link":  parent.absolute_url()},
                {"title": self.context.Title(),
                 "icon":  "fa fa-inbox",
                 "link":  "%s?tab=peers" % deposit_box_link},
                {"title": "Grille d'évaluation",
                 "icon":  "fa fa-th",
                 "link":  "%s/deposit_box_criteria_view" % deposit_box_link}]

    def getCriteriaView(self, user, mode_etudiant):
        # LOG.info("----- getCriteriaView (Start) -----")
        deposit_box = self.context
        is_personnel = self.context.isPersonnel(user, mode_etudiant)
        mode_etudiant = "false" if (not mode_etudiant) and is_personnel else mode_etudiant
        is_grid_access = True if deposit_box.getAccesGrille() or is_personnel else False
        my_view = {"is_anonymous":        self.isAnonymous(),
                   "deposit_box_link":    deposit_box.absolute_url(),
                   "is_depot_actif":      True,
                   "is_correction_actif": True,
                   "is_personnel":        is_personnel,
                   "mode_etudiant":       mode_etudiant,
                   "is_grid_access":      is_grid_access}

        my_view["criteria_dict"] = deposit_box.getCriteriaDict()
        my_view["criteria_order"] = deposit_box.getCriteriaOrder()
        my_view["comment_dict"] = {"0": "Aucun",
                                   "1": "Optionnel",
                                   "2": "Obligatoire"}

        my_view["is_authorized_deposit"] = deposit_box.isDepotActif()
        if my_view["is_authorized_deposit"] == 3:
            my_view["is_authorized_deposit"] = 0
            my_view["is_authorized_deposit_text"] = "Dans le profil évaluation par les pairs les \"dates limite de dépôts et d'évaluation\" sont obligatoires."
        else:
            now = DateTime(DateTime()).strftime("%Y/%m/%d %H:%M")
            date_depot = DateTime(deposit_box.getDateDepot()).strftime("%Y/%m/%d %H:%M")
            if now >= date_depot:
                my_view["is_depot_actif"] = False
            date_correction = DateTime(deposit_box.getDateCorrection()).strftime("%Y/%m/%d %H:%M")
            if now >= date_correction:
                my_view["is_correction_actif"] = False
        # LOG.info("----- getCriteriaView (End) -----")

        return my_view



## Controller Python Script "add_pod_video_script"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=
##title=
##

REQUEST = context.REQUEST
if "video_id" in REQUEST:
    user_id = REQUEST["member_id"]
    video_id_list = [REQUEST["video_id"]]
else:
    form = context.REQUEST.form
    user_id = form["member_id"]
    video_id_list = form["video_ids"]

pod_videos_folder = context.getMySubSpaceFolder(user_id, "Video")
for video_id in video_id_list:
    video_properties = context.searchElasticsearch(type_search="video", term_search=video_id)
    object_id = "Externe-%s-%s" % (video_properties["owner"], video_id)
    object_video = getattr(pod_videos_folder, object_id, None)
    if not object_video:
        pod_videos_folder.invokeFactory(type_name='JalonRessourceExterne', id=object_id)
        object_video = getattr(pod_videos_folder, object_id)
        param = {"Title":                video_properties["title"],
                 "TypeRessourceExterne": "Video",
                 "Videourl":             video_properties["full_url"],
                 "Description":          video_properties["text"],
                 "Lecteur":              video_properties["iframe"],
                 "Videoauteur":          video_properties["owner"],
                 "Videoauteurname":      video_properties["owner_full_name"],
                 "Videothumbnail":       video_properties["thumbnail"]}
        object_video.setProperties(param)

context.REQUEST.RESPONSE.redirect(context.absolute_url())



## Controller Python Script "check_course_password_script"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=
##title=Inscrire un étudiant à un cours après vérification du mot de passe
##

REQUEST = context.REQUEST

context.addPasswordStudent(REQUEST["member_id"])

context.REQUEST.RESPONSE.redirect(context.absolute_url())



## Script (Python) "mail_password"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=
##title=Mail a user's password
##
from Products.CMFPlone import PloneMessageFactory as pmf
from AccessControl import Unauthorized

REQUEST = context.REQUEST
try:
    response = context.portal_registration.mailPassword(REQUEST.get('userid'), REQUEST)
    redirection = "mail_password_confirm"
except ValueError, e:
    try:
        msg = pmf(str(e))
    except Unauthorized:
        # If we are not allowed to tell the user, what is wrong, he
        # should get an error message and contact the admins
        raise e
    context.plone_utils.addPortalMessage(msg)
    response = context.mail_password_form()
    redirection = "mail_password_form"
return REQUEST.RESPONSE.redirect("%s/%s" % (context.absolute_url(), redirection))


##Python Script "add_course_training_offer_script"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=
##title=
##

form = context.REQUEST.form
if form.has_key("elements"):
    context.addCourseTrainingOffer(form["elements"])

context.REQUEST.RESPONSE.redirect(context.absolute_url())



# Controller Python Script "undo_deposit_box_criteria_script"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=
##title=
##

param = {}
form = context.REQUEST.form

criteria_dict = dict(context.getCriteriaDict())

criteria_id = form["criteria_id"]
criteria_dict[criteria_id]["coefficient"] = 0
context.setCriteriaDict(criteria_dict)

context.regenerateAverage()

if "deposit_box_criteria_view" in context.REQUEST.HTTP_REFERER:
    redirection = "%s/deposit_box_criteria_view?criteria_id=%s" % (context.absolute_url(), criteria_id)
else:
    redirection = "%s?tab=peers" % context.absolute_url()

if context.REQUEST.HTTP_X_REQUESTED_WITH != 'XMLHttpRequest':
    context.REQUEST.RESPONSE.redirect(redirection)
else:
    return redirection


## Controller Python Script "send_mail_students_script"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=
##title=
##

request = context.REQUEST
form = request.form

if form["type"] in ["etape", "ue", "uel", "groupe"]:
    listeEtudiants = context.getListeEtudiants(form["code"], form["type"])
    for etudiant in listeEtudiants:
        if etudiant["EMAIL_ETU"]:
            form["a"] = etudiant["EMAIL_ETU"]
            context.envoyerMail(form)
    form["a"] = form["de"]
    context.envoyerMail(form)

request.RESPONSE.redirect(context.absolute_url())



## Controller Python Script "getGoogleAnalytics_script"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=authMember
##title=getGoogleAnalytics_script
##

from Products.CMFCore.utils import getToolByName

# context = context
# authMember = authMember

portal_jalon_properties = getToolByName(context, "portal_jalon_properties")
ga_properties = portal_jalon_properties.getPropertiesGA()
if not ga_properties["activer_ga"]:
    return ""

if authMember.has_role(["Anonymous", "Manager"]):
    return ""

codeDeb = ["<script type='text/javascript'>"]
codeDeb.append("var _gaq = _gaq || [];")
codeDeb.append("_gaq.push(['_setAccount', '*id_account*']);")
codeDeb.append("_gaq.push(['_setDomainName', '*id_domain*']);")
codeDeb.append("_gaq.push(['_trackPageview']);")

codeFin = [""]
codeFin.append("(function() {")
codeFin.append("  var ga = document.createElement('script'); ga.type = 'text/javascript'; ga.async = true;")
codeFin.append("  ga.src = ('https:' == document.location.protocol ? 'https://ssl' : 'http://www') + '.google-analytics.com/ga.js';")
codeFin.append("  var s = document.getElementsByTagName('script')[0]; s.parentNode.insertBefore(ga, s);")
codeFin.append("})();")
codeFin.append("")
codeFin.append("</script>")

code = codeDeb[:]
portal = context.portal_url.getPortalObject()

code[2] = code[2].replace("*id_account*", ga_properties["ga_id_account"])
code[3] = code[3].replace("*id_domain*", ga_properties["ga_id_domain"])

variable = []

roles = authMember.getRoles()
try:
    roles.remove("Member")
except:
    pass
try:
    roles.remove("Authenticated")
except:
    pass
variable.append("_gaq.push(['_setCustomVar',")
variable.append("           1,")
variable.append("           'User Type',")
# Si roles[0] ne fonctionne pas, c'est certainement que le ldap n'est pas bien configuré dans acl_users
variable.append("           '%s'," % roles[0])
variable.append("           1")
variable.append("           ]);")
variable.append("_gaq.push(['_setCustomVar',")
variable.append("           2,")
variable.append("           'User Id',")
variable.append("           '%s'," % portal.mes_etudiants.gaEncodeTexte(ga_properties["ga_cryptage"], authMember.getId()))
variable.append("           1")
variable.append("           ]);")

if context.Type() == "Jalon Cours":
    variable.append("_gaq.push(['_setCustomVar',")
    variable.append("           3,")
    variable.append("           'Id Cours',")
    variable.append("           '%s'," % context.getId())
    variable.append("           3")
    variable.append("           ]);")
if variable:
    code.extend(variable)

code.extend(codeFin)
return "\n".join(code)



# -*- coding: utf-8 -*-

from Products.Five import BrowserView

from jalon.content.content import jalon_utils
from jalon.content import contentMessageFactory as _

from DateTime import DateTime

from logging import getLogger
LOG = getLogger('[CourseStatisticsView]')
"""
# Log examples :
# LOG.info('info message')
"""


class CourseStatisticsView(BrowserView):

    def __init__(self, context, request):
        # LOG.info("----- Init -----")
        BrowserView.__init__(self, context, request)
        self.context = context
        self.request = request

    def getBreadcrumbs(self):
        portal = self.context.portal_url.getPortalObject()
        return [{"title": _(u"Mes cours"),
                 "icon":  "fa fa-university",
                 "link":  "%s/mes_cours" % portal.absolute_url()},
                {"title": self.context.Title(),
                 "icon":  "fa fa-book",
                 "link":  self.context.absolute_url()},
                {"title": "Statistiques",
                 "icon":  "fa fa-bar-chart",
                 "link":  "%s/course_statistics_view" % self.context.absolute_url()}]

    def getRequest(self):
        return self.request

    def getPageView(self, user, mode_etudiant):
        # LOG.info("----- getPageView -----")
        is_personnel = self.context.isPersonnel(user, mode_etudiant)
        mode_etudiant = "false" if (not mode_etudiant) and is_personnel else mode_etudiant

        my_view = {"is_personnel":  is_personnel,
                   "mode_etudiant": mode_etudiant}

        onglet = self.request.get("onglet", "1")
        onglets = []
        cours_url = self.context.absolute_url()
        onglets.append({"href":      "%s/course_statistics_view?onglet=1" % cours_url,
                        "css_class": " selected" if onglet == "1" else "",
                        "icon":      "fa-info",
                        "text":      "Généralités"})
        onglets.append({"href":      "%s/course_statistics_view?onglet=2" % cours_url,
                        "css_class": " selected" if onglet == "2" else "",
                        "icon":      "fa-bar-chart",
                        "text":      "Fréquentation"})
        onglets.append({"href":      "%s/course_statistics_view?onglet=3" % cours_url,
                        "css_class": " selected" if onglet == "3" else "",
                        "icon":      "fa-files-o",
                        "text":      "Supports pédagogiques"})
        onglets.append({"href":      "%s/course_statistics_view?onglet=4" % cours_url,
                        "css_class": " selected" if onglet == "4" else "",
                        "icon":      "fa-random",
                        "text":      "Activités"})

        my_view["onglets"] = onglets
        my_view["onglet_view"] = self.getOngletView(onglet, is_personnel)

        if onglet == "1":
            my_view["course_actions"] = my_view["onglet_view"]["course_actions"]
        return my_view

    def getOngletView(self, onglet, is_personnel):
        # LOG.info("----- getOngletView -----")
        if onglet == "1":
            return self.getIndicateursGeneralitesView(is_personnel)
        if onglet == "2":
            return self.getIndicateursFrequentationView()
        if onglet == "3":
            return self.getIndicateursRessourcesView()
        if onglet == "4":
            return self.getIndicateursActivitiesView()

    def getElementsCoursByType(self):
        # LOG.info("----- getElementsCoursByType -----")
        element_by_type_dict = {"Titre":          {"elements_count": 0, "elements_list": [], "elements_dict": {}},
                                "TexteLibre":     {"elements_count": 0, "elements_list": [], "elements_dict": {}},
                                "Fichiers":       {"elements_count": 0, "elements_list": [], "elements_dict": {}},
                                "Webconference":  {"elements_count": 0, "elements_list": [], "elements_dict": {}},
                                "Externes":       {"elements_count": 0, "elements_list": [], "elements_dict": {}},
                                "Video":          {"elements_count": 0, "elements_list": [], "elements_dict": {}},
                                "VOD":            {"elements_count": 0, "elements_list": [], "elements_dict": {}},
                                "Presentations":  {"elements_count": 0, "elements_list": [], "elements_dict": {}},
                                "BoiteDepot":     {"elements_count": 0, "elements_list": [], "elements_dict": {}},
                                "AutoEvaluation": {"elements_count": 0, "elements_list": [], "elements_dict": {}},
                                "Examen":         {"elements_count": 0, "elements_list": [], "elements_dict": {}},
                                "SalleVirtuelle": {"elements_count": 0, "elements_list": [], "elements_dict": {}},
                                "Forum":          {"elements_count": 0, "elements_list": [], "elements_dict": {}},
                                "Forums":         {"elements_count": len(self.context.forum.objectIds()), "elements_list": [], "elements_dict": {}},
                                "TermeGlossaire": {"elements_count": 0, "elements_list": [], "elements_dict": {}},
                                "Bibliographie":  {"elements_count": len(self.context.getBibliographie()), "elements_list": [], "elements_dict": {}}}
        element_cours_dict = self.context.getCourseItemProperties()
        for element_id in element_cours_dict.keys():
            type_element = element_cours_dict[element_id]["typeElement"]
            if type_element in ["File", "Image", "Page"]:
                type_element = "Fichiers"
            if type_element in ["Lien web", "Lecteur exportable", "Ressource bibliographique", "Catalogue BU", "Video", "VOD"]:
                type_element = "Externes"
            if type_element in ["Presentations sonorisees", "Sonorisation"]:
                type_element = "Presentations"
            element_by_type_dict[type_element]["elements_count"] += 1
            element_by_type_dict[type_element]["elements_list"].append(element_id)
            element_by_type_dict[type_element]["elements_dict"][element_id] = element_cours_dict[element_id]["titreElement"]
        return element_by_type_dict

    def getIndicateursGeneralitesView(self, is_personnel):
        # LOG.info("----- getIndicateursGeneralitesView -----")
        course_training_offer_list = self.context.getCourseTrainingOffer()
        course_training_offer_students = 0
        for course_training_offer in course_training_offer_list:
            course_training_offer_students = course_training_offer_students + int(course_training_offer["nb_etu"])
        course_training_offer = len(course_training_offer_list)

        course_actions = [{"course_actions_id":           "course_to",
                           "course_actions_icon":         "fa fa-university fa-fw",
                           "course_actions_text":         "Offre(s) de formation",
                           "course_actions_list":         [],
                           "course_actions_registration": "%s (%s étu.)" % (course_training_offer, course_training_offer_students),
                           "is_course_password":          False}]
        course_actions.append({"course_actions_id":           "course_nr",
                               "course_actions_icon":         "fa fa-users fa-fw",
                               "course_actions_text":         "Inscription(s) nominative(s)",
                               "course_actions_list":         [],
                               "course_actions_registration": len(self.context.getGroupe()),
                               "is_course_password":          False})
        course_actions.append({"course_actions_id":           "course_er",
                               "course_actions_icon":         "fa fa-envelope-o fa-fw",
                               "course_actions_text":         "Invitation(s) par courriel",
                               "course_actions_list":         [],
                               "course_actions_registration": len(self.context.getInvitations()),
                               "is_course_password":          False})
        course_actions.append({"course_actions_id":           "course_pr",
                               "course_actions_icon":         "fa fa-key fa-fw",
                               "course_actions_text":         "Accès par mot de passe",
                               "course_actions_list":         [],
                               "course_actions_registration": len(self.context.getInscriptionsLibres()),
                               "is_course_password":          self.context.getLibre() and is_personnel,
                               "course_password":             self.context.getLienMooc()})
        course_actions.append({"course_actions_id":           "course_cr",
                               "course_actions_icon":         "fa fa-users fa-fw",
                               "course_actions_text":         "Lecteur(s) enseignant(s)",
                               "course_actions_list":         [],
                               "course_actions_registration": len(self.context.getCoLecteurs()),
                               "is_course_password":          False})

        nb_element_by_type = {"Titre":          0,
                              "TexteLibre":     0,
                              "Fichiers":       0,
                              "Webconference":  0,
                              "Externes":       0,
                              "Video":          0,
                              "VOD":            0,
                              "Presentations":  0,
                              "BoiteDepot":     0,
                              "AutoEvaluation": 0,
                              "Examen":         0,
                              "SalleVirtuelle": 0,
                              "Forum":          0,
                              "Forums":         len(self.context.forum.objectIds()),
                              "TermeGlossaire": 0,
                              "Bibliographie":  len(self.context.getBibliographie())}
        element_cours = self.context.getCourseItemProperties()
        course_map = self.context.getCourseMapList()
        for element_id in course_map:
            element = element_cours[element_id]
            type_element = element["typeElement"]
            if type_element in ["File", "Image", "Page"]:
                type_element = "Fichiers"
            if type_element in ["Lien web", "Lecteur exportable", "Ressource bibliographique", "Catalogue BU", "Video", "VOD"]:
                type_element = "Externes"
            if type_element in ["Presentations sonorisees", "Sonorisation"]:
                type_element = "Presentations"
            nb_element_by_type[type_element] += 1
        return {"macro":               "indicateurs_generalite",
                "created":             jalon_utils.getLocaleDate(self.context.created(), "%d/%m/%Y &agrave; %H:%M"),
                "modified":            jalon_utils.getLocaleDate(self.context.modified(), "%d/%m/%Y &agrave; %H:%M"),
                "nb_element_by_type":  nb_element_by_type,
                "course_actions":      course_actions}

    def getIndicateursFrequentationView(self):
        # LOG.info("----- getIndicateursFrequentationView -----")
        graph = ""
        requete = self.context.getConsultationByCoursByUniversityYearByDate(None, False, False).all()
        # LOG.info(requete)
        if requete:
            requete_dict = {}
            for ligne in requete:
                try:
                    requete_dict[ligne[0]].append({"consultations": ligne[1], "public": ligne[2]})
                except:
                    requete_dict[ligne[0]] = [{"consultations": ligne[1], "public": ligne[2]}]
            try:
                requete_dict[ligne[0]].sort(lambda x, y: cmp(x["public"], y["public"]))
            except:
                pass
            graph = self.context.genererGraphIndicateurs(requete_dict)

        frequentation_graph = ""
        requete2 = self.context.getConsultationByCoursByUniversityYearByDate(None, True, "Etudiant").all()
        # LOG.info(requete2)
        if requete2:
            requete_dict = dict(requete2)
            frequentation_graph = self.context.genererFrequentationGraph(requete_dict)
            # LOG.info(frequentation_graph)

        return {"macro":               "indicateurs_frequentation",
                "year":                "%s/%s" % (DateTime().year() - 1, DateTime().year()),
                "graph":               graph,
                "frequentation_graph": frequentation_graph}

    def getIndicateursRessourcesView(self):
        # LOG.info("----- getIndicateursRessourcesView -----")
        box = self.request.get("box", "1")
        box_dict = {"1": "Fichiers",
                    "2": "Presentations sonorisees",
                    "3": "Externes",
                    "4": "Webconference",
                    "5": "Video",
                    "6": "VOD"}
        cours_url = self.context.absolute_url()
        box_list = [{"css_class": "button radius%s" % (" selected" if box == "1" else " off"),
                     "link_url":  "%s/course_statistics_view?onglet=3&box=1" % cours_url,
                     "icon":      "fa-files-o",
                     "link_text": "Fichiers"},
                    {"css_class": "button radius%s" % (" selected" if box == "2" else " off"),
                     "link_url":  "%s/course_statistics_view?onglet=3&box=2" % cours_url,
                     "icon":      "fa-microphone",
                     "link_text": "Présentations sonorisées"},
                    {"css_class": "button radius%s" % (" selected" if box == "3" else " off"),
                     "link_url":  "%s/course_statistics_view?onglet=3&box=3" % cours_url,
                     "icon":      "fa-external-link",
                     "link_text": "Ressources externes"},
                    {"css_class": "button radius%s" % (" selected" if box == "4" else " off"),
                     "link_url":  "%s/course_statistics_view?onglet=3&box=4" % cours_url,
                     "icon":      "fa-headphones",
                     "link_text": "Webconférences"},
                    {"css_class": "button radius%s" % (" selected" if box == "5" else " off"),
                     "link_url":  "%s/course_statistics_view?onglet=3&box=5" % cours_url,
                     "icon":      "fa-youtube-play",
                     "link_text": "Vidéos"},
                    {"css_class": "button radius%s" % (" selected" if box == "6" else " off"),
                     "link_url":  "%s/course_statistics_view?onglet=3&box=6" % cours_url,
                     "icon":      "fa-video-camera",
                     "link_text": "VOD"}]
        element_id = self.request.get("element_id", None)
        thead_th_list = [{"data-sort": "nb_cons_month_before",
                          "th_title":  "les consultations du mois précédent",
                          "th_text":   "Mois précédent"},
                         {"data-sort": "nb_cons_month",
                          "th_title":  "les consultations du mois courant",
                          "th_text":   "Mois courant"},
                         {"data-sort": "nb_cons_year",
                          "th_title":  "les consultations sur l'année en cours",
                          "th_text":   "Année en cours"}]
        if not element_id:
            thead_th_list.insert(0, {"data-sort": "title",
                                     "th_title":  "le titre du fichier",
                                     "th_text":   "Titre"})
            elements_consultation = []
            elements_cours = self.getElementsCoursByType()
            if box_dict[box] in elements_cours:
                elements_consultation = self.context.getConsultationElementsByCours(elements_cours[box_dict[box]]["elements_list"], elements_cours[box_dict[box]]["elements_dict"])
                # LOG.info(len(elements_consultation))
                # LOG.info(elements_consultation)
            indicateurs_ressources_view = {"macro":                 "indicateurs_ressources",
                                           "box_list":              box_list,
                                           "second_macro":          "display_all_ressources",
                                           "thead_th_list":         thead_th_list,
                                           "elements_consultation": elements_consultation,
                                           "box_url":               "%s/course_statistics_view?onglet=3&box=%s" % (self.context.absolute_url(), box)}
        else:
            thead_th_list.insert(0, {"data-sort": "public",
                                     "th_title":  "le type de public",
                                     "th_text":   "Public"})
            graph = ""
            requete = self.context.getConsultationByElementByCoursByUniversityYearForGraph(element_id).all()
            # LOG.info(requete)
            if requete:
                requete_dict = {}
                for ligne in requete:
                    try:
                        requete_dict[ligne[0]].append({"consultations": ligne[1], "public": ligne[2]})
                    except:
                        requete_dict[ligne[0]] = [{"consultations": ligne[1], "public": ligne[2]}]
                try:
                    requete_dict[ligne[0]].sort(lambda x, y: cmp(x["public"], y["public"]))
                except:
                    pass
                graph = self.context.genererGraphIndicateurs(requete_dict)
            indicateurs_ressources_view = {"macro":                 "indicateurs_ressources",
                                           "element_title":         self.context.getCourseItemProperties(element_id)["titreElement"],
                                           "box_list":              box_list,
                                           "second_macro":          "display_ressource",
                                           "thead_th_list":         thead_th_list,
                                           "elements_consultation": self.context.getConsultationByElementByCours(element_id),
                                           "graph":                 graph,
                                           "box_url":               "%s/course_statistics_view?onglet=3&box=%s" % (self.context.absolute_url(), box)}
        return indicateurs_ressources_view

    def getIndicateursActivitiesView(self):
        """Fournit les indicateurs des Activites."""
        # LOG.info("----- getIndicateursActivitiesView -----")
        box = self.request.get("box", "1")
        box_dict = {"1": "BoiteDepot",
                    "2": "AutoEvaluation",
                    "3": "Examen",
                    "4": "SalleVirtuelle"}
        cours_url = self.context.absolute_url()
        box_list = [{"css_class": "button radius%s" % (" selected" if box == "1" else " off"),
                     "link_url":  "%s/course_statistics_view?onglet=4&box=1" % cours_url,
                     "icon":      "fa-inbox",
                     "link_text": "Boite de dépôts"},
                    {"css_class": "button radius%s" % (" selected" if box == "2" else " off"),
                     "link_url":  "%s/course_statistics_view?onglet=4&box=2" % cours_url,
                     "icon":      "fa-gamepad",
                     "link_text": "Auto-évaluation WIMS"},
                    {"css_class": "button radius%s" % (" selected" if box == "3" else " off"),
                     "link_url":  "%s/course_statistics_view?onglet=4&box=3" % cours_url,
                     "icon":      "fa-graduation-cap",
                     "link_text": "Examen WIMS"},
                    {"css_class": "button radius%s" % (" selected" if box == "4" else " off"),
                     "link_url":  "%s/course_statistics_view?onglet=4&box=4" % cours_url,
                     "icon":      "fa-globe",
                     "link_text": "Salle virtuelle"}]
        element_id = self.request.get("element_id", None)
        thead_th_list = [{"data-sort": "nb_cons_month_before",
                          "th_title":  "les consultations du mois précédent",
                          "th_text":   "Mois précédent"},
                         {"data-sort": "nb_cons_month",
                          "th_title":  "les consultations du mois courant",
                          "th_text":   "Mois courant"},
                         {"data-sort": "nb_cons_year",
                          "th_title":  "les consultations sur l'année en cours",
                          "th_text":   "Année en cours"}]
        if not element_id:
            thead_th_list.insert(0, {"data-sort": "title",
                                     "th_title":  "le titre du fichier",
                                     "th_text":   "Titre"})
            elements_consultation = []
            elements_cours = self.getElementsCoursByType()
            if box_dict[box] in elements_cours:
                elements_consultation = self.context.getConsultationElementsByCours(elements_cours[box_dict[box]]["elements_list"], elements_cours[box_dict[box]]["elements_dict"])
            indicateurs_activites_view = {"macro":                 "indicateurs_ressources",
                                          "box_list":              box_list,
                                          "second_macro":          "display_all_ressources",
                                          "thead_th_list":         thead_th_list,
                                          "elements_consultation": elements_consultation,
                                          "box_url":               "%s/course_statistics_view?onglet=4&box=%s" % (self.context.absolute_url(), box)}
        else:
            thead_th_list.insert(0, {"data-sort": "public",
                                     "th_title":  "le type de public",
                                     "th_text":   "Public"})
            graph = ""
            requete = self.context.getConsultationByElementByCoursByUniversityYearForGraph(element_id).all()
            # LOG.info(requete)
            if requete:
                requete_dict = {}
                for ligne in requete:
                    try:
                        requete_dict[ligne[0]].append({"consultations": ligne[1], "public": ligne[2]})
                    except:
                        requete_dict[ligne[0]] = [{"consultations": ligne[1], "public": ligne[2]}]
                try:
                    requete_dict[ligne[0]].sort(lambda x, y: cmp(x["public"], y["public"]))
                except:
                    pass
                graph = self.context.genererGraphIndicateurs(requete_dict)
            indicateurs_activites_view = {"macro":                "indicateurs_activites",
                                          "element_title":         self.context.getCourseItemProperties(element_id)["titreElement"],
                                          "box_list":              box_list,
                                          "second_macro":          "display_ressource",
                                          "thead_th_list":         thead_th_list,
                                          "elements_consultation": self.context.getConsultationByElementByCours(element_id),
                                          "graph":                 graph,
                                          "box_url":               "%s/course_statistics_view?onglet=4&box=%s" % (self.context.absolute_url(), box)}
        return indicateurs_activites_view



## Controller Python Script "add_course_glossary_term_script"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=
##title=Ajoute une activité au cours
##

form = context.REQUEST.form

if form["title"] in ["", " ", None] or form["description"] in ["", " ", None]:
    redirection = "%s/create_and_add_course_glossary_term_form" % context.absolute_url()
else:
    redirection = context.absolute_url()
    user_id = form["user_id"]
    portal = context.portal_url.getPortalObject()
    portal_workflow = portal.portal_workflow
    folder_object = context.getMySubSpaceFolder(user_id, "Glossaire", portal)

    external_id = "Glossaire-%s-%s" % (user_id, DateTime().strftime("%Y%m%d%H%M%S"))
    folder_object.invokeFactory(type_name="JalonTermeGlossaire", id=external_id)
    external_object = getattr(folder_object, external_id)

    external_object.setProperties({"Title":       form["title"],
                                   "Description": form["description"]})

    context.addMySpaceItemGlossary(folder_object, external_id, "TermeGlossaire", user_id)

    #context.REQUEST.RESPONSE.redirect(context.absolute_url())

context.REQUEST.RESPONSE.redirect(redirection)



## Script (Python) "course_delete_item_script"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=
##title=Supprime ou détache un element ou sous-element du cours
##

# context = context

form = context.REQUEST.form

redirection = context.absolute_url()
if context.meta_type == "JalonCours":
    context.deleteCourseMapItem(form["item_id"], None)
    context.deleteCourseActuality(form["item_id"])
else:
    if "tab" not in form:
        form["tab"] = "documents"

    if form["tab"] == "exercices":
        context.detachExercice(form["item_id"], form["item_order"])
    else:
        context.detachDocument(form["item_id"])

    redirection = "%s?tab=%s" % (context.absolute_url(), form["tab"])

context.REQUEST.RESPONSE.redirect(redirection)



## Controller Python Script "dupliquer_cours"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=
##title=Dupliquer un cours
##

request = context.REQUEST
course_id = request.form["course_id"]
course_user_folder = context.getCourseUserFolder(request.form["user_id"])

course_user_folder.dupliquerCours(course_id, course_user_folder)

request.RESPONSE.redirect("%s?tab=%s" % (context.absolute_url(), request.form["tab"]))



## Controller Python Script "directUrl"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=
##title=
##

return context.REQUEST.RESPONSE.redirect(context.getRemoteUrl())


## Controller Python Script "edit_deposit_box_skill_script"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=
##title=
##

form = context.REQUEST.form

competences = dict(context.getCompetences())
for competence in form["competences"]:
    if competence["titre"]:
        competences[competence["id"]]["titre"] = competence["titre"]
    if competences[competence["id"]]["evaluation"] == "note":
        competences[competence["id"]]["note_max"] = competence["note_max"]
        competences[competence["id"]]["note_partielle"] = competence["note_partielle"]
        competences[competence["id"]]["note_acquise"] = competence["note_acquise"]
    competences[competence["id"]]["description"] = competence["description"]

context.setCompetences(competences)
context.REQUEST.RESPONSE.redirect("%s?tab=skills" % context.absolute_url())


## Script (Python) "caslogin"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=
##title=CAS Login
##
from Products.PythonScripts.standard import url_quote

request = container.REQUEST

portal = context.portal_url.getPortalObject()
plugin = portal.acl_users.cas_unice

if plugin.casServerUrlPrefix:

    pod_action = "/video_edit/"
    jalon_action = str(traverse_subpath[0])
    if jalon_action in ["edit", "delete"]:
        video_slug = context.getVideourl().split("/")[-2]
        pod_action = "/video_%s/%s/" % (jalon_action, video_slug)

    service = url_quote("%s/accounts/cas/login/?next=%s" % (context.getUrlServeurElasticsearch(), pod_action))

    url = "".join([plugin.getLoginURL(), '?service=', service])
    request.RESPONSE.redirect(url, lock=1)
